
/* array_funcs.c        Metaprat       21st Aug 92    */

/* A file containing some commonly used array procedures.  I've avoided  */
/* using the _alist efuns, although they would be faster.  This is since */
/* they are a fairly recent innovation, and I wish to make the houses as */
/* generally compatible as possible.                                     */


search_array(arr, str) {
   /* searches an unsorted flat array, and returns true if the str is */
   /* matched with any items in the array, false otherwise.           */
   int indx;

   if ((!arr) || (!pointerp(arr)) || (sizeof(arr) < 1)) {
      return 0;
   }
   for (indx = 0; indx < sizeof(arr); indx ++) {
      if (arr[indx] == str) {
         return 1;
      }
   }
   return 0;
}


remove_array_item(arr, str) {
   /* returns the unsorted flat array arr, with *all* occurrences of */
   /* element str removed if it was found.  The array will be in the */
   /* same order as it was received.                                 */
   int    indx;
   string new_arr;

   if ((!arr) || (!pointerp(arr)) || (sizeof(arr) < 1)) {
      return ({ });
   }
   new_arr = ({ });
   for (indx = 0; indx < sizeof(arr); indx ++) {
      if (arr[indx] != str) {
         new_arr = new_arr + ({ arr[indx] });
      }
   }
   return new_arr;
}



search_paired_array(arr, str) {
   /* search a flat array in the format:                           */
   /*   ({ index1, item1, index2, item2, ..... indexn, itemn })    */
   /* returning itemX if indexX matched the str provided.          */
   /* An example of this is the 'items' array in /room/room.c      */
   /* Note that confusion may result if the itemX is zero.         */
   int counter;

   if ((!arr) || (!pointerp(arr)) || (sizeof(arr) < 2)) {
      return;
   }
   for (counter = 0; counter < sizeof(arr) - 1; counter += 2) {
      if (arr[counter] == str) {
         return arr[counter + 1];
      }
   }
   return;
}


remove_paired_array_item(arr, str) {
   /* return a flat array in the above format, excluding any item  */
   /* pairs whose index matched the str provided.                  */
   int counter;
   int tmp_arr;

   if ((!arr) || (!pointerp(arr)) || (sizeof(arr) < 2)) {
      return ({ });
   }
   tmp_arr = ({ });
   for (counter = 0; counter < sizeof(arr) - 1; counter += 2) {
      if (arr[counter] != str) {
         tmp_arr = tmp_arr + ({ arr[counter], arr[counter + 1] });
      }
   }
   return tmp_arr;
}


