
 /* house.c   V2.1      Metaprat       6th April 1992  */

 /* This is the players house.  Various household options may be specified */
 /* by they player, as well as the decor.  Entry, and creation/restoration */
 /* is controlled by the house_key, which the player is given when they    */
 /* buy the house.                                                         */

 /* The main role of this file is to hold the house data, and permit       */
 /* manipulations on it.                                                   */


inherit "/players/metaprat/houses/house_room";

#include "house_defines.h"


/* STATIC variables (not restored/saved) */
static string nullstring;
static string owner_name;      /* Supplied by the house key  */
static object rooms;           /* Array of pointers to the room objects  */
static string mort_exit_desc;  /* exit description lines for mortals and */
static string wiz_exit_desc;   /* wizards.  (eg, "There are 3 exits:..)  */

/* potential exit directions (not restored/saved) */
static string valid_cardinals;
static string valid_specials;
static string valid_wizards;
static string recorded_directions;


/* DYNAMIC variables (restored and saved) */
string rooms_desc;      /* long descriptions of all rooms, in array */
string rooms_short;     /* short descs of all rooms, in array       */
string rooms_items;     /* items array of all rooms, in array       */
string furnishings;     /* furnishings array of all rooms, in array */
int    rooms_x;         /* array of x position of rooms (east= +ve) */
int    rooms_y;         /* array of y position of rooms (south +ve) */
int    rooms_z;         /* array of z position of rooms (up positive)*/

/* Exits for south and west are taken from the neighbouring rooms,    */
/* this means that the data structure is proof against paired errors. */

string exits_special;   /* string[room][which_exit]               */
string exits_wizard;    /* string[room][which_exit]               */

int    exits_north;     /* for details of the exit statuses, see  */
int    exits_ne;        /* house_exit.c, which deals with the     */
int    exits_east;      /* state transition table.  Status = 0 is */
int    exits_se;        /* no exit.                               */
int    exits_up;



/* house initialisation & creation code */
/* reset         - called when the object is cloned, and every 20 mins */
/* rejig_data    - compacts used rooms into arrays with no gaps in     */
/* restore_stats - reads the datafile into this object                 */
/* save_stats    - writes the datafile from this object's variables    */


reset(arg) {
   int indx;
   int something_found;

   if (arg) {
      if (owner_name && (!find_player(owner_name))) {
         /* Ok, the following section is to conserve memory.     */
         /* See if anyone is in the house.                       */
         for (indx = (sizeof(rooms) - 1); indx >= 0; indx --) {
            if ((rooms[indx]) && (first_inventory(rooms[indx]))) {
               something_found = 1;
            }
         }
         /* If the house is empty, then make the whole lot go BOOM */
         if (!something_found) {
            self_destruct();
         }
      }
      return;
   }
   valid_cardinals     =
      ({ "north",     "south",     "east",      "west",
         "northeast", "southwest", "southeast", "northwest",
         "up",        "down" });
   valid_specials      =
      ({ "church", "guild", "shop", "post", "pub" });
   valid_wizards       =
      ({ "workroom", "castle", "wizboard" });
   recorded_directions =
      ({ "north", "northeast", "east", "southeast", "up" })   +
      valid_specials  +  valid_wizards;

   if (DEBUGLEVEL > 1) {
      log_file(LOGFILE, "House: house created, ");
      if (this_player()) {
         log_file(LOGFILE, "for " + MYCNAME + ".\n");
      }
      else {
         log_file(LOGFILE, "no player name known.\n");
      }
   }
   set_light(1);

   /* make the restored vars into arrays. Probably unnecessary. */
   rooms_desc    = allocate(MAXNUMROOMS);
   rooms_short   = allocate(MAXNUMROOMS);
   rooms_items   = allocate(MAXNUMROOMS);
   furnishings   = allocate(MAXNUMROOMS);
   rooms_x       = allocate(MAXNUMROOMS);
   rooms_y       = allocate(MAXNUMROOMS);
   rooms_z       = allocate(MAXNUMROOMS);
   exits_north   = allocate(MAXNUMROOMS);
   exits_east    = allocate(MAXNUMROOMS);
   exits_ne      = allocate(MAXNUMROOMS);
   exits_se      = allocate(MAXNUMROOMS);
   exits_special = allocate(MAXNUMROOMS);
   exits_wizard  = allocate(MAXNUMROOMS);
   exits_up      = allocate(MAXNUMROOMS);

   /* set up the rooms table. */
   rooms     = allocate(MAXNUMROOMS);
   rooms[0]  = this_object();
   main_hall = this_object();
}



/* define shorthand routine for moving an item from V to T, being */
/* cautious about the size of the T array, and avoiding +=        */
#define MVVAR(V,T,C) if (sizeof(V)>C) { T=T+({V[C]}); } else { T=T+({0}); }


rejig_data() {
   /* close up gaps in the rooms data. */
   int    counter;
   int    count_rooms;
   string tmp_rooms_desc;
   string tmp_rooms_short;
   string tmp_rooms_items;
   string tmp_furnishings;
   int    tmp_rooms_x;
   int    tmp_rooms_y;
   int    tmp_rooms_z;
   string tmp_exits_special;
   string tmp_exits_wizard;
   int    tmp_exits_north;
   int    tmp_exits_ne;
   int    tmp_exits_east;
   int    tmp_exits_se;
   int    tmp_exits_up;


   /* work out if we want to rejig the data items */
   count_rooms = 1;
   for (counter = 1; counter < sizeof(rooms_desc); counter ++) {
      if (rooms_desc[counter]) {
         count_rooms += 1;
      }
   }

   /* now check that all the arrays are the same size as the number */
   /* of rooms                                                      */
   if ((sizeof(rooms_desc)    == count_rooms) &&
       (sizeof(rooms_short)   == count_rooms) &&
       (sizeof(rooms_items)   == count_rooms) &&
       (sizeof(furnishings)   == count_rooms) &&
       (sizeof(rooms_x)       == count_rooms) &&
       (sizeof(rooms_y)       == count_rooms) &&
       (sizeof(rooms_z)       == count_rooms) &&
       (sizeof(exits_special) == count_rooms) &&
       (sizeof(exits_wizard)  == count_rooms) &&
       (sizeof(exits_north)   == count_rooms) &&
       (sizeof(exits_ne)      == count_rooms) &&
       (sizeof(exits_east)    == count_rooms) &&
       (sizeof(exits_se)      == count_rooms) &&
       (sizeof(exits_up)      == count_rooms)) {
      return;
   }

   tmp_rooms_desc    = ({ });
   tmp_rooms_short   = ({ });
   tmp_rooms_items   = ({ });
   tmp_furnishings   = ({ });
   tmp_rooms_x       = ({ });
   tmp_rooms_y       = ({ });
   tmp_rooms_z       = ({ });
   tmp_exits_special = ({ });
   tmp_exits_wizard  = ({ });
   tmp_exits_north   = ({ });
   tmp_exits_ne      = ({ });
   tmp_exits_east    = ({ });
   tmp_exits_se      = ({ });
   tmp_exits_up      = ({ });

   /* copy across the data item by item */
   for (counter = 0; counter < sizeof(rooms_desc); counter ++) {
      if ((counter == 0) || (rooms_desc[counter])) {
         MVVAR(rooms_desc,    tmp_rooms_desc,    counter);
         MVVAR(rooms_short,   tmp_rooms_short,   counter);
         MVVAR(rooms_items,   tmp_rooms_items,   counter);
         MVVAR(furnishings,   tmp_furnishings,   counter);
         MVVAR(rooms_x,       tmp_rooms_x,       counter);
         MVVAR(rooms_y,       tmp_rooms_y,       counter);
         MVVAR(rooms_z,       tmp_rooms_z,       counter);
         MVVAR(exits_special, tmp_exits_special, counter);
         MVVAR(exits_wizard,  tmp_exits_wizard,  counter);
         MVVAR(exits_north,   tmp_exits_north,   counter);
         MVVAR(exits_ne,      tmp_exits_ne,      counter);
         MVVAR(exits_east,    tmp_exits_east,    counter);
         MVVAR(exits_se,      tmp_exits_se,      counter);
         MVVAR(exits_up,      tmp_exits_up,      counter);
      }
   }
   /* moves the rejigged data back into the main variables */
   rooms_desc    = tmp_rooms_desc;
   rooms_short   = tmp_rooms_short;
   rooms_items   = tmp_rooms_items;
   furnishings   = tmp_furnishings;
   rooms_x       = tmp_rooms_x;
   rooms_y       = tmp_rooms_y;
   rooms_z       = tmp_rooms_z;
   exits_special = tmp_exits_special;
   exits_wizard  = tmp_exits_wizard;
   exits_north   = tmp_exits_north;
   exits_ne      = tmp_exits_ne;
   exits_east    = tmp_exits_east;
   exits_se      = tmp_exits_se;
   exits_up      = tmp_exits_up;
   save_stats();
}





restore_stats(str) {
   /* NOTE: This code assumes the ability to be able to save and restore */
   /* datafiles which include arrays.  This is not present in early, and */
   /* obsolete parsers.                                                  */
   if (DEBUGLEVEL > 1) {
      log_file(LOGFILE,
         "House: reading " + HOUSES_DIR + str + ", at " +
         ctime(time()) + ".\n");
   }
   if (!restore_object(HOUSES_DIR + str)) {
      log_file(LOGFILE,
         "House: restore_stats(" + str + ") failed. " +
         "Using " + HOUSES_DIR + "NEW_OWNER at time: " +
         ctime(time()) + ".\n");
      restore_object(HOUSES_DIR + "NEW_OWNER");
      (MASTERHOUSE)->register_new_house(str);
      save_stats();
   }
   rejig_data();
   rooms          = allocate(sizeof(rooms_desc));
   mort_exit_desc = allocate(sizeof(rooms_desc));
   wiz_exit_desc  = allocate(sizeof(rooms_desc));
   rooms[0]       = this_object();
   main_hall      = this_object();
   gather_room_information(0);    /* set up main hall descriptions */
}



save_stats() {
   /* NOTE: This code assumes the ability to be able to save and restore */
   /* datafiles which include arrays.  This is not present in early, and */
   /* obsolete parsers.                                                  */
   if (DEBUGLEVEL > 1) {
      log_file(LOGFILE,
         "House: saving " + HOUSES_DIR + owner_name + ", " +
         ctime(time()) + ".\n");
   }
   save_object(HOUSES_DIR + owner_name);
}



/* get_ and set_ functions, used to access the variables in this object.  */
/*   _owner                  - sets / returns the house owner             */
/*   _rooms_items            - sets / returns the items array for a room  */
/*   _rooms_short            - sets / returns the short for a room        */
/*   _rooms_long             - sets / returns the long for a room         */
/*   _rooms_furnishings      - sets / returns furnishing for a room       */
/*   _exit_status            - sets / returns status of an exit in a room */
/*   _room_index  (get only) - get room index for a room, given position  */

set_owner(str) {
   owner_name = str;
   (MASTERHOUSE)->register_house(owner_name, this_object());
}


query_owner(str) {
   return owner_name;
}



get_rooms_items(indx) {
   return rooms_items[indx];
}


set_rooms_items(indx, new_items) {
   rooms_items[indx] = new_items;
}


get_rooms_short(indx) {
   return rooms_short[indx];
}


set_rooms_short(indx, new_short) {
   rooms_short[indx] = "" + new_short;
}


get_rooms_long(indx) {
   return rooms_desc[indx];
}


set_rooms_long(indx, new_long) {
   rooms_desc[indx] = "" + new_long;
}


get_rooms_furnishings(indx) {
   return furnishings[indx];
}


set_rooms_furnishings(indx, new_furnishings) {
   furnishings[indx] = new_furnishings;
}


/* code for DIRECTIONS is here. */


set_exit_status(indx, which_door, new_exit_status) {
   int    room_index;
   string which_dir;

   if (!CHKSIZE(indx, 0, sizeof(exits_north) - 1)) { return; }
   room_index = indx;
   which_dir  = "" + which_door;  /* just in case its pass-by-reference */


   if (!search_array(recorded_directions, which_dir)) {
      room_index = direction_room(room_index, which_dir);
      which_dir = opposite_direction(which_dir);
   }
   if (which_dir == "north")          {
      exits_north[room_index] = new_exit_status;
   }
   else if (which_dir == "northeast") {
      exits_ne[room_index] = new_exit_status;
   }
   else if (which_dir == "east")      {
      exits_east[room_index] = new_exit_status;
   }
   else if (which_dir == "southeast") {
      exits_se[room_index] = new_exit_status;
   }
   else if (which_dir == "up")        {
      exits_up[room_index] = new_exit_status;
   }

   /* for the special & wizard exits, the door status is either DOOR_NONE or */
   /* DOOR_ONEWAY.  If we get a change request to DOOR_NONE, we remove it,   */
   /* or otherwise it's added to the appropriate array.                      */
   else if (search_array(valid_specials, which_dir)) {
      if (new_exit_status == DOOR_NONE) {
         exits_special[room_index] =
            remove_array_item(exits_special[room_index], which_dir);
      }
      else if (!search_array(exits_special[room_index], which_dir)) {
         exits_special[room_index] = exits_special[room_index] + ({ which_dir });
      }
   }
   else if (search_array(valid_wizards, which_dir)) {
      if (new_exit_status == DOOR_NONE) {
         exits_wizard[room_index] =
            remove_array_item(exits_wizard[room_index], which_dir);
      }
      else if (!search_array(exits_wizard[room_index], which_dir)) {
         exits_wizard[room_index] = exits_wizard[room_index] + ({ which_dir });
      }
   }
}



get_exit_status(indx, which_door) {
   int    room_index;
   string which_dir;

   if (!CHKSIZE(indx, 0, sizeof(exits_north) - 1)) { return; }
   room_index = indx;
   which_dir  = "" + which_door;  /* just in case it uses call-by-reference */

   if (valid_cardinal(which_dir)) {
      if (!search_array(recorded_directions, which_dir)) {
         room_index = direction_room(indx, which_dir);
         if (room_index < 0) {
            /* no such exit exists */
            return DOOR_NONE;
         }
         which_dir = opposite_direction(which_dir);
      }
      if (which_dir == "north")          { return exits_north[room_index]; }
      else if (which_dir == "east")      { return exits_east[room_index];  }
      else if (which_dir == "northeast") { return exits_ne[room_index]   ; }
      else if (which_dir == "southeast") { return exits_se[room_index]   ; }
      else if (which_dir == "up")        { return exits_up[room_index]   ; }
   }

   /* special and wizard doors are fixed to either be DOOR_NONE (no such */
   /* exit), or DOOR_ONEWAY, if its present in the appropriate array.    */
   else if (search_array(exits_special[room_index], which_dir)) {
      return DOOR_ONEWAY;
   }
   else if (search_array(exits_wizard[room_index], which_dir)) {
      return DOOR_ONEWAY;
   }
   return DOOR_NONE;
}



extract_direction(str) {
   string discard1;
   string discard2;

   if (valid_direction(str)) {
      return str;
   }
   if (sscanf(str, "%s north"    , discard1) == 1) { return "north";     }
   if (sscanf(str, "%s east"     , discard1) == 1) { return "east";      }
   if (sscanf(str, "%s south"    , discard1) == 1) { return "south";     }
   if (sscanf(str, "%s west"     , discard1) == 1) { return "west";      }
   if (sscanf(str, "%s southwest", discard1) == 1) { return "southwest"; }
   if (sscanf(str, "%s southeast", discard1) == 1) { return "southeast"; }
   if (sscanf(str, "%s northeast", discard1) == 1) { return "northeast"; }
   if (sscanf(str, "%s northwest", discard1) == 1) { return "northwest"; }
   if (sscanf(str, "%s up"       , discard1) == 1) { return "up";        }
   if (sscanf(str, "%s down"     , discard1) == 1) { return "down";      }
   if (sscanf(str, "north %s"    , discard1) == 1) { return "north";     }
   if (sscanf(str, "east %s"     , discard1) == 1) { return "east";      }
   if (sscanf(str, "south %s"    , discard1) == 1) { return "south";     }
   if (sscanf(str, "west %s"     , discard1) == 1) { return "west";      }
   if (sscanf(str, "northeast %s", discard1) == 1) { return "northeast"; }
   if (sscanf(str, "southeast %s", discard1) == 1) { return "southeast"; }
   if (sscanf(str, "southwest %s", discard1) == 1) { return "southwest"; }
   if (sscanf(str, "northwest %s", discard1) == 1) { return "northwest"; }
   if (sscanf(str, "up %s"       , discard1) == 1) { return "up";        }
   if (sscanf(str, "down %s"     , discard1) == 1) { return "down";      }
   return 0;
}


valid_cardinal(str) {
   return search_array(valid_cardinals, str);
}


valid_special(str) {
   return search_array(valid_specials, str);
}


valid_wizard(str) {
   return search_array(valid_wizards, str);
}


valid_direction(str) {
   return search_array(valid_cardinals + valid_specials + valid_wizards, str);
}


get_valid_directions() {
   return valid_cardinals + valid_specials + valid_wizards;
}



opposite_direction(which_dir) {
   if (which_dir == "north")     { return "south";     }
   if (which_dir == "south")     { return "north";     }
   if (which_dir == "east")      { return "west";      }
   if (which_dir == "west")      { return "east";      }
   if (which_dir == "northeast") { return "southwest"; }
   if (which_dir == "southwest") { return "northeast"; }
   if (which_dir == "northwest") { return "southeast"; }
   if (which_dir == "southeast") { return "northwest"; }
   if (which_dir == "up")        { return "down";      }
   if (which_dir == "down")      { return "up";        }
   /* The following directions are unimplemented, but included for */
   /* the sake of future enhancement.                              */
   if (which_dir == "in")        { return "out";       }
   if (which_dir == "out")       { return "in";        }
   if (which_dir == "enter")     { return "leave";     }
   if (which_dir == "leave")     { return "enter";     }
   return 0;
}


direction_xoff(which_dir) {
   /* return the offset in the x-axis for any given direction */
   if (search_array( ({ "northeast", "east", "southeast" }), which_dir)) {
      return 1;
   }
   if (search_array( ({ "northwest", "west", "southwest" }), which_dir)) {
      return -1;
   }
}


direction_yoff(which_dir) {
   /* return the offset in the y-axis for any given direction */
   if (search_array( ({ "southwest", "south", "southeast" }), which_dir)) {
      return 1;
   }
   if (search_array( ({ "northwest", "north", "northeast" }), which_dir)) {
      return -1;
   }
}


direction_zoff(which_dir) {
   /* return the offset in the z-axis for any given direction */
   if (search_array( ({ "out", "up", "leave" }), which_dir)) {
      return 1;
   }
   if (search_array( ({ "in",  "down", "enter" }), which_dir)) {
      return -1;
   }
}


build_exit_description(indx) {
   string exits_list_mort;
   string exits_list_wiz;
   int    counter;
   int    tmp_exit_status;

   /* Make the exit description for mortals. */
   exits_list_mort = exits_special[indx];
   exits_list_wiz  = exits_special[indx] + exits_wizard[indx];
   for (counter = 0; counter < sizeof(valid_cardinals); counter ++) {
      tmp_exit_status = get_exit_status(indx, valid_cardinals[counter]);
      if (tmp_exit_status != DOOR_NONE) {
         if (tmp_exit_status != DOOR_HIDDEN) {
            exits_list_mort = exits_list_mort + ({ valid_cardinals[counter] });
            exits_list_wiz  = exits_list_wiz +  ({ valid_cardinals[counter] });
         }
         else {
            exits_list_wiz = exits_list_wiz +
                             ({ "(" + valid_cardinals[counter] + ")" });
         }
      }
   }
   if (sizeof(exits_list_mort) == 0) {
      mort_exit_desc[indx] = "    There are NO obvious exits";
   }
   else if (sizeof(exits_list_mort) == 1) {
      mort_exit_desc[indx] = "    There is one obvious exit: ";
   }
   else {
      mort_exit_desc[indx] = "    There are " +
         textual_number(sizeof(exits_list_mort)) + " obvious exits: ";
   }
   mort_exit_desc[indx] += implode(exits_list_mort, ", ");

   /* Now we create the exit description for wizards. */
   if (sizeof(exits_list_mort) == sizeof(exits_list_wiz)) {
      wiz_exit_desc[indx] = mort_exit_desc[indx];
      return;
   }
   wiz_exit_desc[indx] =  "    There are " +
      textual_number(sizeof(exits_list_wiz)) + " obvious exits: " +
      implode(exits_list_wiz, ", ");
}



textual_number(arg) {
   if (!intp(arg)) {
      return "NO";
   }
   if (arg > 17) {
      return "" + arg;
   }
   return
      ({"NO",     "one",      "two",      "three",   "four",    "five",
        "six",    "seven",    "eight",    "nine",    "ten",     "eleven",
        "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen"})
      [arg];
}


file_name_exit(indx, str) {
   int door_status;

   /* do the cardinal exits */
   if (valid_cardinal(str)) {
      door_status = get_exit_status(indx, str);
      if (door_status == DOOR_NONE) {
         return 0;
      }
      if (!check_door(indx, door_status, str)) {
         return 1;
      }
      return file_name(find_room(rooms_x[indx] + direction_xoff(str),
                                 rooms_y[indx] + direction_yoff(str),
                                 rooms_z[indx] + direction_zoff(str)));
   }
   /* special exits */
   if (str == "church")   { return CHURCH; }
   if (str == "guild")    { return GUILD; }
   if (str == "shop")     { return SHOP; }
   if (str == "post")     { return POST; }
   if (str == "pub")      { return PUB; }
   /* Now do the wizard exits. */
   if (!IAMWIZ) {
      return 0;
   }
   if (str == "workroom") { return WORKROOM; }
   if (str == "wizboard") { return WIZBOARD; }
   if (str == "castle")   {
      if (find_object(CASTLE) && environment(find_object(CASTLE))) {
         return file_name(environment(find_object(CASTLE)));
      }
      return CASTLE;
   }
}



check_door(indx, door_status, which_door) {
   string action;
   string new_action;
   int    new_state;

   action = (DOORSTATES)->get_status_property_action(door_status, "enter");
   if (intp(action)) {
      /* don't bother changing the door state, because we know it */
      /* stays the same after an "enter" in any case.             */
      return 1;
   }
   if (sscanf(action, "O:%d:%s", new_state, new_action) == 2) {
      if (MYNAME == GETOWNER) {
         return 1;
      }
      else {
         write(new_action);
         return 0;
      }
   }
   write(action);
   return 0;
}




change_exit_pair(indx, which_door, door_status) {
   int    next_room;
   string next_dir;

   if ((door_status != DOOR_NONE)   &&
       (door_status != DOOR_NORM)   &&
       (door_status != DOOR_L_LOCK) &&
       (door_status != DOOR_L_UNLK) &&
       (door_status != DOOR_L_OPEN) &&
       (door_status != DOOR_O_CLOS) &&
       (door_status != DOOR_O_OPEN) &&
       (door_status != DOOR_PORTAL) &&
       (door_status != DOOR_ONEWAY) &&
       (door_status != DOOR_HIDDEN)) {
      log_file(LOGFILE, "House_room: Tried to set bad exit status.\n");
      return;
   }

   /* now change the status of the door on the opposite side of the wall */
   next_dir = opposite_direction(which_door);
   if (next_dir) {
      next_room = direction_room(indx, which_door);
      if (next_room < 0) {
         write("But there is no room in that direction!\n");
         return;
      }
      change_exit_status(next_room, next_dir, door_status);
   }
   change_exit_status(indx, which_door, door_status);
   MONEY->charge_money(DOORCOST, "door");
   save_stats();
}



change_exit_status(indx, which_door, door_status) {

   if (door_status != DOOR_NONE) {
      notify_room(indx,
         "A team of builders rush in, and do something to the " +
         which_door + " exit.  It\nlooks different.\n");
   }
   else {
      notify_room(indx,
         "A team of demolition experts do something to the " +
         which_door + " exit.  Its gone!\n");
   }
   set_exit_status(indx, which_door, door_status);
   build_exit_description(indx);
}



/* code for manipulating rooms */
/* create_new_room         - make a new room, may increase array sizes       */
/* find_room               - returns an obj num for a room given position    */
/* direction_room          - return indx for a room in a dir from given room */
/* query_room              - return obj number for a room given its indx     */
/* query_room_position     - return X,Y&Z for a room given its index         */
/* notify_room             - tell a room something, given index, & str       */
/* get_room_index          - returns an index for a room given position      */
/* execu_rooms_id          - executes id() on behalf of a room               */
/* execu_rooms_short       - executes short() on behalf of a room            */
/* execu_rooms_long        - executes long() on behalf of a room             */
/* gather_room_information - creates a room given its index                  */
/* self_destruct           - destructs all rooms, move contents to church    */
/* erase_room              - destroy a room, clearing it from the arrays     */
/* move_contents           - move the contents of one object to another      */



create_new_room(newx, newy, newz) {
   int counter;
   int indx;
   /* Add a new room.  Do NOT record it, since it will be recorded by the */
   /* ensuing call to change_exit_pair.                                   */

   indx = -1;
   for (counter = 0; counter < sizeof(rooms_desc); counter ++) {
      if ((!rooms_desc[counter]) || (rooms_desc[counter] == "")) {
         indx = counter;
         break;
      }
   }
   /* if no usable gaps exit, add another entry on the end of */
   /* the various arrays for it.  Checks the max first.       */
   if ((indx < 0) && (sizeof(rooms_desc) < MAXNUMROOMS)) {
      indx = sizeof(rooms_desc);
      rooms         = rooms         + ({ 0 });
      mort_exit_desc= mort_exit_desc+ ({ 0 });
      wiz_exit_desc = mort_exit_desc+ ({ 0 });
      rooms_desc    = rooms_desc    + ({ 0 });
      rooms_short   = rooms_short   + ({ 0 });
      rooms_items   = rooms_items   + ({ 0 });
      furnishings   = furnishings   + ({ 0 });
      rooms_x       = rooms_x       + ({ 0 });
      rooms_y       = rooms_y       + ({ 0 });
      rooms_z       = rooms_z       + ({ 0 });
      exits_special = exits_special + ({ 0 });
      exits_wizard  = exits_wizard  + ({ 0 });
      exits_north   = exits_north   + ({ 0 });
      exits_ne      = exits_ne      + ({ 0 });
      exits_east    = exits_east    + ({ 0 });
      exits_se      = exits_se      + ({ 0 });
      exits_up      = exits_up      + ({ 0 });
   }
   if (indx < 0) {
      write("You have reached the maximum number of rooms.\n");
      return -1;
   }
   /* set the initial values of the room's data to something that */
   /* helps them to decorate it.                                  */
   rooms_desc[indx]    =
"This is a brand new, unfurnished room.  As soon as you see it, you\n" +
"start imagining interesting ways to decorate it.. it could be anything,\n" +
"a cupboard, a conservatory, an orchard, heaven or hell..\n" +
"A small note in the corner says:  'help build/furnish'.\n";
   rooms_short[indx]   = "A brand new room, ready to be decorated";
   furnishings[indx]   = nullstring;
   rooms_items[indx]   = nullstring;
   rooms_x[indx]       = newx;
   rooms_y[indx]       = newy;
   rooms_z[indx]       = newz;
   exits_special[indx] = ({ });
   exits_wizard[indx]  = ({ });
   exits_up[indx]      = 0;
   exits_north[indx]   = 0;
   exits_ne[indx]      = 0;
   exits_east[indx]    = 0;
   exits_se[indx]      = 0;
   save_stats();
   find_room(newx, newy, newz);  /* clone the room, add it to rooms array */
   return indx;
}



find_room(newx, newy, newz) {
   int room_index;

   room_index = get_room_index(newx, newy, newz);
   if ((room_index < 0) || (room_index >= MAXNUMROOMS)) {
      log_file(LOGFILE,
         "House: ERROR, find_room(" + newx + ", " + newy + ", " + newz +
         ") found "+room_index+". Owner = "+owner_name + ",\ntime was: " +
         ctime(time()) + ".\n");
      return main_hall;   /* make exit to main room */
   }
   if (!rooms[room_index]) {
      gather_room_information(room_index);
   }
   return rooms[room_index];
}



direction_room(indx, which_dir) {
   /* Returns index of the room on the other side of the exit.  If no */
   /* such room exists, then returns -1.                              */
   int dest_x;
   int dest_y;
   int dest_z;

   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return -1;
   }
   if (valid_cardinal(which_dir)) {
      dest_x = direction_xoff(which_dir) + rooms_x[indx];
      dest_y = direction_yoff(which_dir) + rooms_y[indx];
      dest_z = direction_zoff(which_dir) + rooms_z[indx];
      return get_room_index(dest_x, dest_y, dest_z);
   }
   return -1;
}



query_room(indx) {
   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return 0;
   }
   return rooms[indx];
}



query_room_position(indx) {
   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return 0;
   }
   if (rooms_desc[indx]) {
      return ({ rooms_x[indx], rooms_y[indx], rooms_z[indx] });
   }
   return 0;
}



notify_room(indx, str) {
   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return;
   }
   if (rooms[indx]) {
      tell_room(rooms[indx], str);
   }
}



get_room_index(newx, newy, newz) {
   int indx;

   if ((newx == 0) && (newy == 0) && (newz == 0)) {
      return 0;    /* just hardwire the check for the main room in */
   }
   for (indx = 1; indx < sizeof(rooms_x); indx ++) {
      if ((newx == rooms_x[indx]) &&
          (newy == rooms_y[indx]) &&
          (newz == rooms_z[indx])) {
         return indx;
      }
   }
   return -1;   /* search failed */
}



execu_rooms_id(indx, str) {
   /* this takes over the operation of id() from the house_room */

   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return 0;
   }
   if (search_paired_array(rooms_items[indx], str)) {
      return 1;
   }
   if (pointerp(furnishings[indx]) &&
       search_array(furnishings[indx], "kitchen")) {
      if (((ALLOW_DRINK) && ((str == "drinks")       ||
                            (str == "cabinet")       ||
                            (str == "drinks cabinet"))) ||
          ((ALLOW_FOOD) && ((str == "cupboard")      ||
                           (str == "ingredients")    ||
                           (str == "food"))))      {
         return 1;
      }
   }
   return (extract_direction(str) != 0);
}



execu_rooms_short(indx) {
   /* this takes over the operation of short() from the house_room */
   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return 0;
   }
   if (!mort_exit_desc[indx]) {
      /* need to build the exits description for the room */
      build_exit_description(indx);
   }
   if (!IAMWIZ) {
      return rooms_short[indx] + ".\n" + mort_exit_desc[indx];
   }
   return rooms_short[indx] + ".\n" + wiz_exit_desc[indx];
}


execu_rooms_long(indx, str) {
   /* this takes over the operation of long() from house_room */
   string tmp_text;
   int    counter;

   if ((indx < 0) || (indx >= sizeof(rooms))) {
      return -1;
   }
   if (str) {
      /* match the specified obj, and describe it */
      tmp_text = search_paired_array( rooms_items[indx], str );
      if (tmp_text) {
         write(tmp_text + ".\n");
      }
      if (pointerp(furnishings[indx])                 &&
          search_array(furnishings[indx], "kitchen")) {
         if ((ALLOW_DRINK) && ((str == "drinks")           ||
                               (str == "cabinet")          ||
                               (str == "drinks cabinet"))) {
            SUSTENANCE->list_items("drink");
            return;
         }
         if ((ALLOW_FOOD) && ((str == "cupboard")    ||
                              (str == "ingredients") ||
                              (str == "food")))       {
            SUSTENANCE->list_items("food");
            return;
         }
      }
      if (extract_direction(str)) {
         write((DOORSTATES)->
                  get_status_property_action(
                     get_exit_status(indx, extract_direction(str)),
                     "exam"));
      }
   }
   else {
      /* describe the room */
      write(get_rooms_long(indx));
      if (!mort_exit_desc[indx]) {
         /* need to build the exits description for the room */
         build_exit_description(indx);
      }
      if (!IAMWIZ) {
         write(mort_exit_desc[indx] + ".\n");
      }
      else {
          write(wiz_exit_desc[indx] + ".\n");
      }
   }
}



static gather_room_information(room_index) {
   /* clones the new room, and sets up the two main pieces of information */
   /* that it needs.  (eg, this object, and also which index it has).     */
   if ((room_index < 0) || (room_index >= sizeof(rooms))) {
      return -1;
   }
   if (!rooms[room_index]) {
      rooms[room_index] = clone_object(ROOMS);
   }
   rooms[room_index]->set_main_hall(this_object());
   rooms[room_index]->set_room_index(room_index);
}


self_destruct() {
   int indx;

   /* destruct the rooms one by one, from the highest to the lowest */
   /* (which is the main_hall).  Move contents to church.           */
   for (indx = sizeof(rooms) - 1; indx >= 0; indx --) {
      if (rooms[indx]) {
         tell_room(rooms[indx],
            "A massive explosion rips through the house, destroying all.\n");
         move_contents(rooms[indx], find_object(CHURCH));
         destruct(rooms[indx]);
      }
   }
}



erase_room(indx) {
   int found_exit;
   int counter;

   if (indx == 0) {
      write("The main room may not be destroyed.\n");
      return 0;
   }
   if ((indx < 1) || (indx >= sizeof(rooms))) {
      write("Rooms index number out of range.\n");
      return 0;
   }

   for (counter = 0;
        counter < sizeof(valid_cardinals);
        counter ++) {
      if (get_exit_status(indx, (valid_cardinals)[counter])) {
         write("There are exits remaining! These must be destroyed first..\n");
         return 0;
      }
   }
   log_file(LOGFILE,
      "house: Room " + indx + " destroyed by " + owner_name + ", at " +
      ctime(time()) + ".\n");
   if (rooms[indx]) {
      tell_room(rooms[indx],
         "A demolition team turn up, throw you out, and smash the room into" +
         "\na dusty pile of rubble.\n");
      move_contents(rooms[indx], this_object());
      destruct(rooms[indx]);
   }
   rooms_desc[indx]    = nullstring;
   rooms_short[indx]   = nullstring;
   rooms_items[indx]   = ({ });
   furnishings[indx]   = nullstring;
   rooms_x[indx]       = 0;
   rooms_y[indx]       = 0;
   rooms_z[indx]       = 0;
   exits_special[indx] = ({ });
   exits_wizard[indx]  = ({ });
   exits_north[indx]   = 0;
   exits_ne[indx]      = 0;
   exits_east[indx]    = 0;
   exits_se[indx]      = 0;
   exits_up[indx]      = 0;
   save_stats();
   return 1;
}


move_contents(origin, destination) {
   /* move the inventory of one object to another object */
   object thisob;
   object nextob;

   thisob = first_inventory(origin);
   while (thisob) {
      nextob = next_inventory(thisob);
      move_object(thisob, destination);
      if (living(thisob)) {
         command("look",thisob);
      }
      thisob = nextob;
   }
}


