
 /* House_builder.c   V2.1      Metaprat       6th April 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


static string new_exit_dir;
static int    line;


/* standard functions */


id(str) {
   return (str == "house builder") || (str == "builder");
}


short() {
   return "house builder";
}


long(str) {
   write("This object allows the house owner to build rooms onto his or\n");
   write("her house.  Commands are:\n");
   write("\tbuild room [dir]\t- build a room in the specified direction.\n");
   write("\tbuild exit [dir]\t- build an exit in the specified direction.\n");
   write("If you don't specify a direction, then you will be asked for\n");
   write("the direction to build in.  If you're building an exit, you will\n");
   write("also be asked for the exit type.\n");
   write("Valid directions:  north, south, east, west, northeast,\n");
   write("   southeast, southwest, northwest, up, and down.\n");
   write("Exits may also go to:  church, shop, guild, pub or post(office).\n");
   write("There are several types of exits, as follows:\n");
   write("\tnormal       \t- a normal exit\n");
   write("\topenable door\t- door which may be opened & closed (by anyone)\n");
   write("\tlockable door\t- locked & unlocked by house owner\n");
   write("\tmagic portal \t- only the house owner may go through it.\n");
   write("\thidden exit  \t- not shown on obvious exits list, nor maps.\n");
   write("Exits going outside the house are always of 'normal' type.\n");
}



toolkit_init() {
   add_action("build", "build");
}



build(str) {
   string room_dir;
   object this_main_hall;

   this_main_hall = environment(this_player())->query_main_hall();
   if (!this_main_hall) {
      write("You are not in a house.\n");
      return 1;
   }
   if (this_main_hall != main_hall) {
      write("You are not in your own house.\n");
      return 1;
   }
   this_room_index = environment(this_player())->query_room_index();
   if (!str) {
      write("Usage: build {room/exit} [direction]\n");
      write("For information, look at the house builder.\n");
      return 1;
   }
   if (str == "room") {
      if (!MONEY->has_money(ROOMCOST)) {
         return 1;
      }
      write("In what direction do you want to build the new room?\n");
      write("[north/south/east/west/northeast/..etc../up/down] : ");
      input_to("build_room");
      return 1;
   }
   if ((sscanf(str, "room %s", room_dir) == 1)  ||
       (sscanf(str, "%s room", room_dir) == 1)) {
      if (main_hall->valid_cardinal(room_dir)) {
         if (!MONEY->has_money(ROOMCOST)) {
            return 1;
         }
         build_room(room_dir);
         return 1;
      }
   }
   if ((str == "door") || (str == "exit")) {
      if (!MONEY->has_money(DOORCOST)) {
         return 1;
      }
      write("In what direction do you want the door?\n");
      write("[" + implode(main_hall->get_valid_directions(),"/") + "] : ");
      input_to("select_exit_dir");
      return 1;
   }
   if ((sscanf(str, "door %s", room_dir) == 1) ||
       (sscanf(str, "exit %s", room_dir) == 1) ||
       (sscanf(str, "%s door", room_dir) == 1) ||
       (sscanf(str, "%s exit", room_dir) == 1)) {
      if (main_hall->valid_direction(room_dir)) {
         if (!MONEY->has_money(DOORCOST)) {
            return 1;
         }
         select_exit_dir(room_dir);
         return 1;
      }
   }
   write("What?  You should specify 'room' or 'door', and an optional\n");
   write("direction  (eg, build door east).\n");
   return 1;
}


static build_room(str) {
   string room_direction;

   if ((str == "abort") || (str == "q") || (str == "quit")) {
      write("You change your mind about building a room.\n");
      return;
   }
   if (!main_hall->valid_cardinal(str)) {
      write("[abort/north/east/..etc.../northeast/...etc.../up/down] : ");
      write("Please enter your selection : ");
      input_to("build_room");
      return;
   }
   create_new_room(str);
}



static select_exit_dir(str) {
   string exit_direction;

   if ((str == "abort") || (str == "q") || (str == "quit")) {
      write("You change your mind about building an exit.\n");
      return;
   }
   if (!main_hall->valid_direction(str)) {
      write("The direction should be one of the following:\n");
      write(implode(main_hall->get_valid_directions(),",") + ".\n");
      write("Please enter your selection : ");
      input_to("select_exit_dir");
      return;
   }
   if (main_hall->get_exit_status(this_room_index, str) == DOOR_ONEWAY) {
      write("But there is already an exit that leads there!\n");
      return;
   }
   if ((main_hall->valid_special(str)) || (main_hall->valid_wizard(str))) {
      main_hall->change_exit_pair(this_room_index, str, DOOR_ONEWAY);
      return;
   }
   else {
      new_exit_dir = str;
      write("Now please select which type of exit you want.\n");
      write("   a  -  normal exit        (recommended)\n");
      write("   b  -  door (open/close)\n");
      write("   c  -  door (open/close/lock)\n");
      write("   d  -  magic portal; only the house owner may use it.\n");
      write("   e  -  hidden exit, not on obvious exits list, nor maps.\n");
      write("[a/b/c/d/e] : ");
      input_to("select_exit_type");
   }
}


static select_exit_type(str) {
   int exit_type;

   if ((str == "abort") || (str == "q") || (str == "quit")) {
      write("You change your mind about building an exit.\n");
      return;
   }
   if ((str != "a") && (str != "b") && (str != "c") && (str != "d") &&
       (str != "e")) {
      write("The type must be one of abort/a/b/c/d/e.\n");
      write("Please enter your selection : ");
      input_to("select_exit_type");
      return;
   }
   if (str == "a") {
      exit_type = DOOR_NORM;
   }
   if (str == "b") {
      exit_type = DOOR_O_CLOS;
   }
   if (str == "c") {
      exit_type = DOOR_L_LOCK;
   }
   if (str == "d") {
      exit_type = DOOR_PORTAL;
   }
   if (str == "e") {
      exit_type = DOOR_HIDDEN;
   }
   main_hall->change_exit_pair(this_room_index, new_exit_dir, exit_type);
}



static create_new_room(which_dir) {
   string next_dir;
   int    indx;
   int    loc_x;
   int    loc_y;
   int    loc_z;

   indx = main_hall->direction_room(this_room_index, which_dir);
   if (indx >= 0) {
      write("Eh??  But that room already exists!\n");
      return;
   }
   indx = 1;
   while (indx < MAXNUMROOMS) {
      if (!main_hall->query_room_position(indx)) {
         break;
      }
      indx ++;
   }
   if ((indx < 0) || (indx >= MAXNUMROOMS)) {
      write("All the rooms you are permitted to build are already built..\n");
      return;
   }
   indx = main_hall->query_room_position(this_room_index);
   loc_x = indx[0] + main_hall->direction_xoff(which_dir);
   loc_y = indx[1] + main_hall->direction_yoff(which_dir);
   loc_z = indx[2] + main_hall->direction_zoff(which_dir);
   write("The builders work away at your new room, and then send you a\n");
   write("bill.  After they leave, a woodworker arrives to fit a door.\n");
   MONEY->charge_money(ROOMCOST, "build");
   main_hall->create_new_room(loc_x, loc_y, loc_z);
   write("The woodworker finishes, and charges you his standard fee.\n");
   main_hall->change_exit_pair(this_room_index, which_dir, DOOR_NORM);
   main_hall->save_stats();
}


