
 /* House_code_gen.c        Metaprat       6th April 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


/* standard functions */


id(str) {
   return (str == "house code generator") ||
          (str == "code generator")       ||
          (str == "generator");
}


short() {
   return "house code generator";
}


long(str) {
   write("This object allows the wizard owner to generate LPC code that\n");
   write("is a reasonable copy of the room.  Exits will be all of the\n");
   write("'normal' type, and there will not be any 'furnish' capabilities\n");
   write("in the generated code.\n");
   write("The syntax is:    generate_lpc <full file name>\n");
   write("The code that is generated will be in the filename specified.\n");
   write("eg)    generate_lpc /players/metaprat/rooms/house_7.c\n");
   write("The best thing to call the eventual file is 'house_' followed\n");
   write("by the index number of the room (its on the map).\n");
}



toolkit_init() {
   if (IAMWIZ) {
      add_action("generate_lpc", "generate_lpc");
   }
}


generate_lpc(str) {
   int    counter;
   string this_line;
   string remainder;
   string filenm;
   string termnm;
   string pathnm;
   string discard;
   string discard2;
   string poss_exits;

   if (!environment(this_player())->confirm_house()) {
      write("You are not in a house room.\n");
      return 1;
   }
   main_hall       = environment(this_player())->query_main_hall();
   this_room_index = environment(this_player())->query_room_index();

   /* Work out the path to the file (pathnm), its terminal name (termnm) */
   /* and the file that is being appended to (filenm).                   */
   if (!str) {
      filenm = "/open/house_" + MYNAME + "_" + this_room_index + ".c";
      termnm = "house_" + this_room_index + ".c";
      pathnm = "/players/" + MYNAME + "/rooms";
   }
   else {
      filenm = str;
      if ((sscanf(str, "/open/%s", discard) == 1) ||
          (sscanf(str, "/players/" + MYNAME + "/%s", discard) == 1)) {
         filenm = str;
         /* build up the directory that the file will be in */
         pathnm = "";
         while (sscanf(filenm, pathnm + "/%s/%s", discard, discard2) == 2) {
            pathnm = pathnm + "/" + discard;
         }
         sscanf(filenm, pathnm + "/%s", termnm);
         if (termnm != ("house_" + this_room_index + ".c")) {
            write("Beware, if you generate other rooms, they will be\n");
            write("using the name 'house_" + this_room_index + ".c' to " +
                  "refer to this one.\n");
         }
      }
      else {
         write("You can only write into /open, or your own directory.\n");
         write("You have to supply the full path, eg)\n");
         write("   generate_lpc /players/" + MYNAME + "/rooms/room17.c\n");
         return 1;
      }
   }
   pathnm = "\"" + pathnm + "/\"";

   write("Generating LPC code for the current room.\n");
   write("Target: " + filenm + "\n");
   write_file(filenm, "/* Metaprat - automatically generated house room */\n");
   write_file(filenm, "\n");
   write_file(filenm, "/* Save as: " + pathnm + termnm + " */\n");
   write_file(filenm, "\n");
   write_file(filenm, "inherit \"/room/room.c\";\n");
   write_file(filenm, "\n");
   write_file(filenm, "#define ROOMSDIR  " + pathnm + "\n");
   write_file(filenm, "#define PREFIX    \"house_\"\n");
   write_file(filenm, "\n");
   write_file(filenm, "/* object declarations for monsters etc go here */\n");
   write_file(filenm, "\n");
   write_file(filenm, "reset(arg) {\n");
   write_file(filenm, "   /* This routine is called with arg == 0 when */\n");
   write_file(filenm, "   /* the room is created, and arg == 1 later.  */\n");
   write_file(filenm, "   if (!arg) {\n");
   write_file(filenm, "      no_castle_flag = 1;\n");
   write_file(filenm, "      set_light(1);\n");
   write_file(filenm, "\n");


   /* code for the directions array */
   write_file(filenm, "      /* Directions are here, and is an array    */\n");
   write_file(filenm, "      /* containing exit destination / exit verb */\n");
   write_file(filenm, "      dest_dir =  ({\n");
   poss_exits = main_hall->get_valid_directions();
   for (counter = 0; counter < sizeof(poss_exits); counter ++) {
      if (main_hall->get_exit_status(this_room_index,
                                     poss_exits[counter]) != DOOR_NONE) {
         if (main_hall->valid_cardinal(poss_exits[counter])) {
            /* Its an exit to another room inside the house */
            write_file(filenm,
                       "         ROOMSDIR + PREFIX + \"" +
                       main_hall->direction_room(this_room_index,
                                                 poss_exits[counter]));
         }
         else {
            /* an exit to a room outside the house */
            write_file(filenm,
                       "         \"" +
                       main_hall->file_name_exit(this_room_index,
                                                 poss_exits[counter]));
         }
         write_file(filenm, "\", \"" + poss_exits[counter] + "\"");
         if (counter < sizeof(poss_exits) - 2) {
           write_file(filenm, ",");
         }
         write_file(filenm, "\n");
      }
   }
   write_file(filenm, "            });\n");


   /* code for the short description */
   write_file(filenm, "      /* Brief room description */\n");
   write_file(filenm, "      short_desc = \"" +
                      main_hall->get_rooms_short(this_room_index) + "\";\n");


   /* Now produce the long description code */
   write_file(filenm, "      /* Description of the room*/\n");
   write_file(filenm, "      long_desc =\n");
   discard = main_hall->get_rooms_long(this_room_index);
   while (sscanf(discard, "%s\n%s", this_line, discard2) == 2) {
      if ((discard2) && (discard2 != "")) {
         write_file(filenm, "\"" + this_line + "\\n\" +\n");
      }
      else {
         write_file(filenm, "\"" + this_line + "\\n\";\n");
      }
      discard = discard2;
   }
   discard2 = "";


   /* Now the items array */
   discard = main_hall->get_rooms_items(this_room_index);
   write_file(filenm, "      /* examinable items */\n");
   write_file(filenm, "      items = ({\n");
   for (counter = 0; counter < sizeof(discard) - 1; counter += 2) {
      write_file(filenm, "\"" + discard[counter] + "\", \"" +
                         discard[counter + 1] + "\"");
      if (counter < sizeof(discard) - 3) {
        write_file(filenm, ",");
      }
      write_file(filenm, "\n");
   }
   write_file(filenm, "      });\n");

   write_file(filenm, "      /* Permanent objects should be cloned here */\n");
   write_file(filenm, "      /* since this is only executed once.       */\n");
   write_file(filenm, "   }\n");
   write_file(filenm, "   /* If you have any monsters or movable objs */\n");
   write_file(filenm, "   /* you want to clone into this room, do it  */\n");
   write_file(filenm, "   /* here, since its executed every reset.    */\n");
   write_file(filenm, "}\n");

   return 1;
}



