
/* house_defines.h      Metaprat       16th April   */

/* various definitions used in the houses */

#define WIZLEVEL     20

/* how much junk goes into /log/LOGFILE */
#define DEBUGLEVEL   1

/* various files used in the houses */
#define METACASTLE   "/players/metaprat/castle"
#define HOUSES_DIR   "players/metaprat/houses/"
#define ROOMS        ("/" + HOUSES_DIR + "house_room")
#define INVITATIONS  ("/" + HOUSES_DIR + "house_invite")
#define MAPS         ("/" + HOUSES_DIR + "house_map")
#define ESTATEAGENT  ("/" + HOUSES_DIR + "estate_agent")
#define HOUSEKEEPER  ("/" + HOUSES_DIR + "house_keeper")
#define HOUSEHELP    ("/" + HOUSES_DIR + "house_help")
#define MONEY        ("/" + HOUSES_DIR + "house_money")
#define SUSTENANCE   ("/" + HOUSES_DIR + "house_sustenance")
#define HOUSEDRINK   ("/" + HOUSES_DIR + "house_drink")
#define INVITATIONS  ("/" + HOUSES_DIR + "house_invite")
#define MASTERHOUSE  ("/" + HOUSES_DIR + "master_house")
#define HOUSEKEY     ("/" + HOUSES_DIR + "house_key")
#define DOORSTATES   ("/" + HOUSES_DIR + "house_exit")
#define HOUSE        ("/" + HOUSES_DIR + "house")

/* toolkits, all inherited from TOOLKIT, and cloned at will. */

#define TOOLKIT      ("/" + HOUSES_DIR + "house_toolkit_obj")
#define BUILDER      ("/" + HOUSES_DIR + "house_builder")
#define CODEGEN      ("/" + HOUSES_DIR + "house_code_gen")
#define EDITDESC     ("/" + HOUSES_DIR + "house_desc_editor")
#define EDITITEM     ("/" + HOUSES_DIR + "house_item_editor")
#define EDITFURN     ("/" + HOUSES_DIR + "house_furnish_edit")
#define DESTROYER    ("/" + HOUSES_DIR + "house_destroyer")
#define ROOMSTATS    ("/" + HOUSES_DIR + "house_stats")


/* Code fragments */
#define GETOWNER     main_hall->query_owner()
#define CHKSIZE(ITEM,LOW,HIGH) ( (ITEM >= LOW) && (ITEM <= HIGH) )
#define MYNAME       this_player()->query_real_name()
#define MYCNAME      this_player()->query_name()
#define IAMWIZ       (this_player()->query_level() >= WIZLEVEL)
#define IAMOWNER     (main_hall && (MYNAME == GETOWNER))

/* logging.  By changing log_file to write_file, can log to your own */
/* directory   (change LOGFILE to include the full path).            */
#define LOGFILE      "metaprat.house"
#define LOGCMD(STR)  log_file(LOGFILE, STR);
#define LOGTHIS(LEV,STR) if (lev <= DEBUGLEVEL) { LOGCMD(STR); }

/* standard locations within the mudlib */
/* used in special & wizard exits, HOUSEKEY, INVITATIONS */
#define STORAGE      "/room/storage"
#define CHURCH       "/room/church"
#define GUILD        "/room/adv_guild"
#define SHOP         "/room/shop"
#define POST         "/room/post"
#define PUB          "/room/pub2"
#define WORKROOM     ("/players/" + MYNAME + "/workroom")
#define CASTLE       ("/players/" + MYNAME + "/castle")
#define WIZBOARD     "/room/adv_inner"


/* timeout period for maps & invites to turn to dust */
#define TIMEOUT      240

/* the number of rooms. Don't make it too high.. 30 is the ABSOLUTE limit */
#define MAXNUMROOMS  20       /* used in several places */

/* maximum size of a description. Should be < max str size & max write size */
#define MAXDESCLEN   1200     /* used in DESCEDIT */
#define MAXDESCLINES 20
#define MAXITEMNUM   10       /* used in ITEMEDIT */
#define MAXITEMLEN   80

/* charges for various building jobs etc.  MONEY file is important. */
#define HOUSECOST    5000     /* used in estate agent */
#define ROOMCOST     2000     /* in BUILDER           */
#define KEEPERCOST   1000     /* not implemented      */
#define FURNISHCOST  800      /* used in FURNISHINGS  */
#define DECORCOST    200      /* used in DESCEDIT     */
#define DOORCOST     400      /* used in BUILDER      */
#define ITEMCOST     50       /* used in ITEMEDIT     */
#define INVITECOST   50       /* used in HOUSEROOM    */
#define INVITEPERMCOST 1000   /* used in HOUSEROOM    */
#define MAPCOST      10       /* used in HOUSEROOM    */

/* Door types, and state.  Main file concerned with door_state is */
/* HOUSEEXIT, which has a state transition table.                 */
#define DOOR_NONE    0
#define DOOR_NORM    1    /* normal exit */
#define DOOR_L_LOCK  10   /* lockable door, 3 state */
#define DOOR_L_UNLK  11
#define DOOR_L_OPEN  12
#define DOOR_O_CLOS  20   /* openable door, 2 state */
#define DOOR_O_OPEN  21
#define DOOR_PORTAL  30   /* owner only exit */
#define DOOR_ONEWAY  40   /* non-cardinal exits */
#define DOOR_HIDDEN  50   /* not listed in obvious dirs, or mapped */



/*      Tuning definitions for the house sustenance code.  The ALLOW_     */
/* definitions control whether the item is allowed to sell drinks or food */
/* at all.  If it is, then the _ALLOW_COFFEE and _ALLOW_HEAL control      */
/* whether sobering drinks, and very high heal drinks are allowed to be   */
/* sold.  Note that they are very expensive in any case.                  */

/*      To fine tune the effectiveness of the food and drinks, set the    */
/* following: _STR_TUNE, controls how alcoholic or how filling the item   */
/* is.  The higher it is, the less they will be able to drink or eat.     */

/*      _COST_TUNE is now expensive it is.  _HEAL_TUNE is how much        */
/* healing the item does when it is first ingested, in addition to the    */
/* normal increase during the drunken period.  Note that food only uses   */
/* the strength to control this, and _HEAL_ is ignored.                   */

/*      Try to stick with the filenames mentioned, since other            */
/* implementations of food & drink may use different calls to set them    */
/* up.  If you use /obj/alco_drink, _allow_coffee and _allow_heal MUST be */
/* false, since it isn't flexible enough to be able to deal with them.    */

#define ALLOW_DRINK             1
#define DRINK_FILE              HOUSEDRINK
#define DRINK_STR_TUNE(ARG)     (ARG * 100 / 100)
#define DRINK_COST_FORMULA(ARG) (4*ARG + ((ARG*ARG)/10))
#define DRINK_COST_TUNE(ARG)    (ARG * 100 / 100)
#define DRINK_HEAL_TUNE(ARG)    (ARG * 100 / 100)
#define DRINK_ALLOW_COFFEE      1
#define DRINK_ALLOW_HEAL        1

#define ALLOW_FOOD              1
#define FOOD_FILE               "/obj/food"
#define FOOD_STR_TUNE(ARG)      (ARG * 100 / 100)
#define FOOD_COST_FORMULA(ARG)  (4*ARG + ((ARG*ARG)/10))
#define FOOD_COST_TUNE(ARG)     (ARG * 100 / 100)


query_drop_castle() {
   /* prevent castles being dropped in this room */
   return 1;
}
