
 /* House_room.c   V2.1      Metaprat       6th April 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


string new_long_desc;
string new_short_desc;
int    line;


/* standard functions */


id(str) {
   return (str == "house desc editor");
}


short() {
   return "house desc editor";
}


long(str) {
   write("This is a house description editor.  It allows the owner of a\n");
   write("house to alter the description of the current room.  Commands\n");
   write("defined are:\n");
   write("\tdecorate \t- decorate the current room\n");
   write("You will be asked to supply a 'short' description, and a 'long'\n");
   write("description.  The short is a single line of text used by brief,\n");
   write("and by the map's index.  The long description should consist of\n");
   write("a series of up to 20 lines, each approximately 70 characters\n");
   write("long.\n");
   write("Commands within the building of the long description are:\n");
   write("\t**         \t- finish the description\n");
   write("\t~q         \t- abort the description\n");
   write("\t!<command> \t- do a command\n");
   write("\t<control>-R\t- redraw line (only on some terminals)\n");
}


toolkit_init() {
   add_action("decorate", "decorate");
   add_action("decorate", "redecorate");
}


decorate() {
   object this_main_hall;
   this_main_hall = environment(this_player())->query_main_hall();
   if (!this_main_hall) {
      write("You are not in a house.\n");
      return 1;
   }
   if (this_main_hall != main_hall) {
      write("You are not in your own house.\n");
      return 1;
   }
   this_room_index = environment(this_player())->query_room_index();
   if (!MONEY->has_money(DECORCOST)) {
      return 1;
   }
   if (DEBUGLEVEL > 1) {
      log_file(LOGFILE,
         "House_room: " + GETOWNER + " decorating room " + this_room_index +
         ", at " +
         ctime(time()) + ".\n");
   }
   say(capitalize(GETOWNER) + " starts redecorating the room.\n");
   write("You start redecorating the room.\n");
   write("Please enter a short, single line, description of the room.\n");
   write("This is used by 'brief', and by the map for its index.\n");
   write("A '.' will be added automatically to the end of the line.\n");
   write("Short>>");
   input_to("get_short_desc");
   return 1;
}


static get_short_desc(str) {
   if (!str || (str == "")) {
      write("No short text supplied.  Aborted.\n");
      line = 1;
      return;
   }
   new_short_desc = str;
   write("Enter full room description.  End with '**', abort with '~q'.\n");
   write("Try to end each line after about 70 characters.\n");
   write(
" >>---------|---------|---------|---------|---------|---------|---------*\n");
   write("1>>");
   new_long_desc = "";
   line = 1;
   input_to("get_desc");
}


static get_desc(str) {
   if (str == "~q") {
      say(capitalize(GETOWNER) + " stops redecorating, with a " +
          "grimace.\n");
      write("You stop redecorating the room, and restore it to its original " +
            "splendor.\n");
      new_long_desc = "";
      line = 1;
      return;
   }
   if ((line > MAXDESCLINES) || (strlen(new_long_desc + str) > MAXDESCLEN)) {
      write("You have exceeded the maximum size for room descriptions.\n");
      write("No more lines can be added.\n");
      str = "**";
   }
   if (str == "**") {
      if (line == 1) {
         write("You decide not to decorate the room after all.\n");
         say(capitalize(GETOWNER)+ " decides not to redecorate after all.\n");
         new_long_desc = "";
         line = 1;
         return;
      }
      write("You step back to take a look at the new decorations.\n");
      say(capitalize(GETOWNER)
          + " finishes redecorating, and takes a look at the result.\n");
      write("----------------------------Short---------------------------\n");
      write(new_short_desc + ".\n");
      write("--------------------------Long-Desc-------------------------\n");
      write(new_long_desc);
      write("------------------------------------------------------------\n");
      write("If this is acceptable, please enter [yY].  If not, the old\n");
      write("room description will be used instead.  : ");
      input_to("check_desc");
      return;
   }
   new_long_desc = new_long_desc + str + "\n";
   line += 1;
   if (line > (MAXDESCLINES - 4)) {
      write("WARNING: near maximum description size (lines).\n");
   }
   else if (strlen(new_long_desc) > ((MAXDESCLEN * 9) / 10)) {
      write("WARNING: near maximum description size (bytes).\n");
   }
   write(line + ">>");
   input_to("get_desc");
   return;
}


static check_desc(str) {
   if ((str == "y") || (str == "Y") || (str == "yes") || (str == "YES")) {
      write("You make the new room decor permanent.\n");
      say(capitalize(GETOWNER) +
          " smiles, and admires the new decoration.\n");
      main_hall->set_rooms_short(this_room_index, new_short_desc);
      main_hall->set_rooms_long(this_room_index, new_long_desc);
      main_hall->save_stats();   /* record the new descriptions */
      new_long_desc = 0;
      new_short_desc = 0;
      line = 0;
      MONEY->charge_money(DECORCOST, "decor");
      return;
   }
   if ((str == "n") || (str == "N") || (str == "no")  || (str == "NO")) {
      write("You look in horror at the decor, and junk the lot.\n");
      say(capitalize(GETOWNER) +
          " looks in horror at the new decor, and junks the lot.\n");
      new_long_desc = 0;
      new_short_desc = 0;
      line = 0;
      return;
   }
   write("Please enter y, yes, n, or no.  : ");
   input_to("check_desc");
}


