
 /* House_destroyer.c   Metaprat       5th August 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


string new_exit_dir;


/* standard functions */


id(str) {
   return (str == "house destroyer") || (str == "destroyer");
}


short() {
   return "house destroyer";
}


long(str) {
   write("This is a house destroyer.  It allows the house owner to destroy\n");
   write("exits or rooms in the house.  Commands are:\n");
   write("\tdestroy room      \t- destroy the room you are standing in.\n");
   write("\tdestroy exit [dir]\t- destroy the specified exit.\n");
   write("All exits must be destroyed before you can destroy the room.\n");
   write("You are given the opportunity to change your mind in both\n");
   write("commands.\n");
   write("If you don't supply a direction when you destroy an exit, you\n");
   write("will be asked which exit you with to destroy.\n");
}


toolkit_init() {
   if (MYNAME == GETOWNER) {
      add_action("destroy",  "destroy");
   }
}



destroy(str) {
   string door_dir;
   object this_main_hall;

   this_main_hall = environment(this_player())->query_main_hall();
   if (!this_main_hall) {
      write("You are not in a house.\n");
      return 1;
   }
   if (this_main_hall != main_hall) {
      write("You are not in your own house.\n");
      return 1;
   }
   this_room_index = environment(this_player())->query_room_index();
   if (!str) {
      write("Usage: destroy {room/exit [direction]}\n");
      write("For more information, look at the house destroyer.\n");
      return 1;
   }
   if (str == "room") {
      say(MYCNAME + " calls in the demolition experts " +
          "to utterly destroy this room.\n");
      write("Are you SURE you wish to utterly destroy this room?  It will\n");
      write("be totally erased from your house.  Also note, if any exits\n");
      write("to other rooms remain, the builders will refuse to demolish\n");
      write("it.  Furthermore, if you have accidently isolated rooms then\n");
      write("there could be serious problems.\n");
      write("Please confirm: [yes/y] to destroy the room : ");
      input_to("destroy_room_confirm");
      return 1;
   }
   if ((str == "door") || (str == "exit")) {
      if (!MONEY->has_money(DOORCOST)) {
         return 1;
      }
      write("Which door do you want to destroy?  Please select from these,\n");
      write("[abort/" + implode(main_hall->get_valid_directions(),"/") +
            "] : ");
      input_to("destroy_door_dir");
      return 1;
   }
   if ((sscanf(str, "door %s", door_dir) == 1) ||
       (sscanf(str, "exit %s", door_dir) == 1) ||
       (sscanf(str, "%s door", door_dir) == 1) ||
       (sscanf(str, "%s exit", door_dir) == 1)) {
      if (main_hall->valid_direction(door_dir)) {
         if (!MONEY->has_money(DOORCOST)) {
            return 1;
         }
         say(MYCNAME + " asks a team of builders to\n" +
             " remove the " + door_dir + " door.\n");
         write("Are you sure you want to destroy the door?  It could leave\n");
         write("some rooms isolated.\n");
         write("Enter [yes/y] to destroy the " + door_dir + " door : ");
         new_exit_dir = door_dir;
         input_to("destroy_door_confirm");
         return 1;
      }
   }
   write("What?  You should specify 'room' or 'door'.  Room always refers\n");
   write("to THIS room, but door has to specify a direction.\n");
   write("Eg),    'destroy room'    or    'destroy door north'.\n");
   write("See 'help destroy' for warnings about this command.\n");
   return 1;
}


static destroy_room_confirm(str) {
   if ((str != "yes") && (str != "y") &&
       (str != "no")  && (str != "n")) {
      write("You must enter [yes/y/no/n] : ");
      input_to("destroy_room_confirm");
      return;
   }
   if ((str == "no") || (str == "n")) {
      write("You change your mind about utterly destroying this room.\n");
      say(MYCNAME + " decides not to utterly destroy " +
          "the room after all.\n");
      return;
   }
   if (this_room_index == 0) {
      write("You may not destroy the entrance.\n");
      say("Demolition experts look around, shake their heads, and leave.\n");
      return;
   }
   if (main_hall->erase_room(this_room_index)) {
      write("You have successfully destroyed the room.\n");
   }
   else {
      write("You failed in your bid to destroy the room.\n");
      say("Demolition experts fall over laughing, and say: April fool!\n");
   }
   return;
}



static destroy_door_dir(str) {
   if ((str == "abort") || (str == "quit") || (str == "q")) {
      write("You change your mind.\n");
      return;
   }
   if (!main_hall->valid_direction(str)) {
      write("The direction must be one of:\n");
      write("[abort/" + implode(main_hall->get_valid_directions(),"/") +
            "] : ");
      input_to("destroy_door_dir");
      return;
   }
   if (main_hall->get_exit_status(this_room_index, str) == DOOR_NONE) {
      write("But there is no exit there anyway!\n");
      return;
   }
   new_exit_dir = str;
   say(MYCNAME + " asks a team of builders to remove\n" +
       " the " + str + " door.\n");
   write("Are you sure you want to destroy the door?  It could leave some\n");
   write("rooms isolated.\n");
   write("Enter [yes/y] to destroy the " + str + " door : ");
   input_to("destroy_door_confirm");
}



static destroy_door_confirm(str) {
   if ((str != "yes") && (str != "y") &&
       (str != "no")  && (str != "n")) {
      write("You must enter [yes/y/no/n] : ");
      input_to("destroy_door_confirm");
      return;
   }
   if ((str == "no") || (str == "n")) {
      write("You change your mind about utterly destroying the door.\n");
      say(MYCNAME + " decides not to utterly destroy " +
          "the " + new_exit_dir + " door\nafter all.\n");
      new_exit_dir = "";
      return;
   }
   write("The builders remove the door, and send you a bill.\n");
   main_hall->change_exit_pair(this_room_index,new_exit_dir, DOOR_NONE);
   new_exit_dir == "";
   /* charge will be done in change_exit_pair */
}



