
/* house_drink.c        Metaprat       10th August 1992 */

/* The standard alco_drink uses 'strength' as both the heal strength and */
/* the alcoholic strength.  Since I need to define both sobering drinks  */
/* and high heal (high cost) drinks, this has to be changed.             */

/* The main changes:   Uses DRINK_COST_FORMULA to get cost             */
/*                     if heal == strength, identical to the standard  */
/*                     low strength drinks add to drink_soft in player */


#include "house_defines.h"
inherit "/obj/alco_drink";


int heal;

set_heal(arg) {
   /* should be called AFTER set_strength */
   heal = arg;
}

query_heal(arg) {
   return heal;
}

set_strength(arg) {
   /* if this is called, and not set_heal(), then the drink behaves */
   /* identically to the standard alco_drink.c                      */
   strength = arg;
   if (!heal) {
      heal  = arg;
   }
}

min_cost() {
   if (heal < strength) {
      return ::min_cost();
   }
   return DRINK_COST_FORMULA(heal);
}


drink(str) {
   object tp;
   string p_name;

   if (heal == strength) {
      return ::drink(str);
   }

   if (!full) {
      return;
   }

   if (!str || !id(str)) {
      return;
   }

   /* strength is used to judge the alcoholic strength of the fluid */
   if (strength / 2 >= 12 && (this_player()->query_level() < 10)) {
      write("You splutter liquid all over the room.\n");
      say(MYCNAME + " tries a " + name + " but coughs " +
          "and\nsplutters all over you.\n");
      full = 0;
      return 1;
   }

   if (strength / 2 >= 8 && this_player()->query_level() < 5) {
      write("You throw it all up.\n");
      say(MYCNAME + " tries to drink a " + name +
          " but throws up.\n");
      full = 0;
      return 1;
   }

   if (!this_player()->drink_alco(strength)) {
      return 1;
   }

   /* Another difference here.  If the drink is weak, then it adds to the */
   /* soft_drink counter (after all, weak alco drinks still have liquid   */
   /* bulk!!)                                                             */
   if (strength < 5) {
      if (!this_player()->drink_soft(2)) {
         return 1;
      }
   }

   full = 0;
   /* and HERE is the main difference.  Why on earth should the heal */
   /* amount be tied so closely to strength?                         */
   this_player()->heal_self(heal);
   return 1;
}


