
 /* House_furnish_editor.c      Metaprat       23rd August 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


static string furnishings;
static string furnish_selected;

#define VALID_FURNISHINGS ({ "kitchen" })


/* standard functions */


id(str) {
   return (str == "house furnish editor") || (str == "furnish editor") ||
          (str == "editor");
}


short() {
   return "house furnish editor";
}


long() {
   write("A house furnish editor.\n");
   write("It allows the owner of a house to modify the 'furnishings'\n");
   write("(capabilities of the room).  For example, if you give the\n");
   write("room a 'kitchen' capability, you can pour drinks, and cook\n");
   write("food in it  (help kitchen).\n");
   write("Commands available:\n");
   write("\tfurnish \t- Puts you into the furnish editor menu.\n");
   write("Once you are there, you are offered the following choices:\n");
   write("\tq       \t- quit furnish menu, loosing new furnish list\n");
   write("\tu       \t- update the room with the new furnish list\n");
   write("\ta <name>\t- add furnishing to the list\n");
   write("\td <name>\t- delete furnishing from the list\n");
   write("\tl       \t- list furnishings in the list\n");
   write("\th or ?  \t- help\n");
   write("The valid furnishings:\n");
   write("\tkitchen\t- as described above.\n");
}


toolkit_init() {
   add_action("furnish", "furnish");
}


write_furnish_menu() {
   write("Furnish menu:\n");
   write(" q        - quit furnish menu, loosing new furnish list\n");
   write(" u        - update the room with the new furnish list\n");
   write(" a <name> - add a furnishing to the list\n");
   write(" d <name> - delete a furnishing from the list\n");
   write(" l        - list the new furnishings for the room\n");
   write(" h or ?   - help\n");
}


furnish(str) {
   object this_main_hall;

   this_main_hall = environment(this_player())->query_main_hall();
   if (!this_main_hall) {
      write("You are not in a house.\n");
      return 1;
   }
   if (this_main_hall != main_hall) {
      write("You are not in your own house.\n");
      return 1;
   }
   this_room_index = environment(this_player())->query_room_index();
   furnishings = main_hall->get_rooms_furnishings(this_room_index);
   if (!furnishings || !pointerp(furnishings)) {
      furnishings = ({ });
   }
   write_furnish_menu();
   list_furnish();
   input_to("edit_furnish");
   return 1;
}



set_furnish_info() {
   object here;

   MONEY->charge_money(FURNISHCOST, "furnish");
   write("You update " + main_hall->get_rooms_short(this_room_index) + ".\n");
   main_hall->set_rooms_furnishings(this_room_index, furnishings);
   main_hall->save_stats();
   this_room_index = -1;
   furnishings = ({ });
   return 1;
}


edit_furnish(str) {
   if (!MONEY->has_money(FURNISHCOST)) {
      return 1;
   }
   if (!str || (str == "")) {
      write_furnish_menu();
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   if (lower_case(str) == "q") {
      write("Leaving furnish menu, discarding new furnish list.\n");
      this_room_index = -1;
      furnishings = ({ });
      return 1;
   }
   if (lower_case(str) == "a") {
      write("Please enter the name of the furnishing to add : ");
      input_to("select_add_furnish");
      return 1;
   }
   if (sscanf(lower_case(str), "a %s", furnish_selected) == 1) {
      add_furnish();
      return 1;
   }
   if (lower_case(str) == "d") {
      write("Please enter the name of the furnishing to delete : ");
      input_to("select_delete_furnish");
      return 1;
   }
   if (sscanf(lower_case(str), "d %s", furnish_selected) == 1) {
      delete_furnish();
      return 1;
   }
   if (lower_case(str) == "l") {
      list_furnish();
      input_to("edit_furnish");
      return 1;
   }
   if (lower_case(str) == "u") {
      set_furnish_info();
      return 1;
   }
   if ((lower_case(str) == "h") || (str == "?")) {
      write("To add the 'kitchen' capability to a room:\n");
      write(" > furnish\n");
      write(" (the menu displayed here)\n");
      write(" Select: a kitchen               (adding to list)\n");
      write(" Select: l                       (list furnish in list)\n");
      write(" [1]  kitchen.\n");
      write(" Select: u                       (update the room)\n");
      write(" You are charged " + FURNISHCOST + " coins.\n");
      write(" You update A large, spacious kitchen.\n");
      write(" > look at drinks cabinet\n");
      write(" (lots of drinks displayed)\n");
      write(" > look at igredients\n");
      write(" (lots of food displayed)\n");
      input_to("edit_furnish");
      return 1;
   }
   write("Didn't understand furnish menu selection.\n");
   write_furnish_menu();
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



select_add_furnish(str) {
   if (str && (str != "")) {
      furnish_selected = str;
      add_furnish();
   }
   else {
      write("No furnish name provided.  Returning to furnish menu.\n");
      write("Select: ");
      input_to("edit_furnish");
   }
}



select_delete_furnish(str) {
   if (str && (str != "")) {
      furnish_selected = str;
      delete_furnish();
   }
   else {
      write("No furnish name provided.  Returning to furnish menu.\n");
      write("Select: ");
      input_to("edit_furnish");
   }
}



add_furnish() {
   if (!search_array(VALID_FURNISHINGS, furnish_selected)) {
      write("No such furnishing option exists.  You may select from the\n");
      write("following:\n");
      write("[" + implode(VALID_FURNISHINGS, "/") + "]\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   if (search_array(furnishings, furnish_selected)) {
      write("That has already been selected.\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   furnishings = furnishings + ({ furnish_selected });
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



delete_furnish() {
   int tmp_size;

   if (sizeof(furnishings) <= 0) {
      write("No furnishings in list.\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   tmp_size = sizeof(furnishings);
   furnishings  = remove_array_item(furnishings, furnish_selected);
   if (tmp_size == sizeof(furnishings)) {
      write("Nothing removed from furnishing list.\n");
   }
   else {
      write("" + (tmp_size - sizeof(furnishings)) + " furnishing removed.\n");
   }
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



list_furnish() {
   int counter;

   if (sizeof(furnishings) <= 0) {
      write("No furnishings in list.\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   for (counter = 0; counter < sizeof(furnishings); counter ++) {
      write("[" + (counter + 1)  + "]\t" + furnishings[counter] + ".\n");
   }
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



