
 /* house_invite.c       Metaprat     6th April 1992  */

 /* An object that is given to another player, to invite them to the first */
 /* player's house.  It has a short time-out period, and only works from   */
 /* the church, shop, guild, post office, pub, or another house.           */

 /* A 'permanent' invitation is more expensive, autoloads, and does not */
 /* self-destruct.                                                      */


#define FILENAME "/players/metaprat/houses/house_invite"
#include "house_defines.h"

string  invitee;
int     permanent;

id(str) {
   return (str == "invitation")  ||  (str == short())     ||
          (lower_case(str) == invitee + "'s invitation")  ||
          (lower_case(str) == invitee + "s invitation")   ||
          (lower_case(str) == invitee + "'s invite")      ||
          (lower_case(str) == invitee + "s invite")       ;
}


short() {
   if (!invitee) {
      return "A blank invitation";
   }
   if (permanent) {
      return "permanent invitation to visit " +
             capitalize(invitee) + "'s house";
   }
   return "invitation to visit " + capitalize(invitee) + "'s house";
}


long() {
   write("This is an invitation to visit " + capitalize(invitee) + "'s\n");
   if (permanent) {
      write("house.  This is a beautiful invitation, written on a strong\n");
      write("sheet of vellum, and should last for many visits.  The\n");
   }
   else {
      write("house.  It will only last for a short time, so quickly go to\n");
      write("the church (or any house), if you want to accept it.  The\n");
   }
   write("command is 'accept " + capitalize(invitee) + "s invitation'.\n");
}


get() {
   return 0;     /* may not be picked up */
}


drop() {
   return 1;     /* may not be dropped */
}


query_weight() {
   return 0;     /* weightless  */
}


query_value() {
   return 0;     /* may not be sold */
}



init() {
   if (environment(this_object()) == this_player()) {
      add_action("accept","accept");
   }
   if (IAMWIZ) {
      add_action("invite_stats","invite_stats");
   }
}



accept(str) {
   string discard;
   string here;
   object house;

   if ((str) && (!id(str))) {
      return 0;
   }
   if (!IAMWIZ) {
      here = file_name(environment(this_player())) + "junk" ;
      if ((!sscanf("/" + here, CHURCH + "%s", discard)) &&
          (!sscanf("/" + here, GUILD  + "%s", discard)) &&
          (!sscanf("/" + here, SHOP   + "%s", discard)) &&
          (!sscanf("/" + here, POST   + "%s", discard)) &&
          (!sscanf("/" + here, PUB    + "%s", discard)) &&
          (!sscanf("/" + here, STORAGE+ "%s", discard)) &&
          (!sscanf("/" + here, HOUSE  + "%s", discard))) {
         write("You can only accept the invitation if you are in the\n");
         write("church, guild, post office, pub, shop or in a house.\n");
         return 1;
      }
   }
   house = find_destination();
   if (!house) {
      write("Sorry, the house is not currently occupied.\n");
      return 1;
   }
   write("You visit " + capitalize(invitee) + ".\n");
   if (find_player(invitee)) {
      tell_object(find_player(invitee),
         MYCNAME + " accepts your kind invitation.\n");
   }
   else {
      write("Warning, the person you wish to visit is not in the game.\n");
   }
   this_player()->move_player("to " + capitalize(invitee) + "'s house#" +
                              file_name(find_destination()));
   if (!permanent) {
      remove_call_out("timeout");
      destruct(this_object());
   }
   return 1;
}



timeout() {
   if (permanent) {
      return;
   }
   if (DEBUGLEVEL > 2) {
      log_file(LOGFILE,
         "house_invite: " + MYCNAME + " fails to decide " +
         " about " + invitee + "'s invitation in time.\n");
   }
   if (find_player(invitee)) {
      tell_object(find_player(invitee),
         MYCNAME +
         " fails to respond to your invitation within the time limit.\n");
   }
   say("You fail to respond quickly enough to the invitation, and it\n" +
       "turns to dust with old age.\n");
   destruct(this_object());
}



invite_stats() {
   write("invitation statistics:\n");
   write("File name:  " + file_name(this_object()) + ".\n");
   write("Invitee:    " + invitee + ".\n");
   write("Debug lev:  " + DEBUGLEVEL + ".\n");
   if (permanent) {
      write("The invitation is permanent.\n");
   }
   return 1;
}



set_invitee(str, perm) {
   invitee = str;
   if (perm) {
      permanent = perm;
   }
   else {
      call_out("timeout",TIMEOUT);
   }
}



query_auto_load() {
   if (permanent) {
      return FILENAME + ":" + invitee;
   }
}


init_arg(arg) {
   permanent = 1;
   invitee = arg;
}


find_destination() {
   /* only allow the invite holder to clone the house if its both a  */
   /* permanent invite, and the house is registered with the master. */
   object house;

   house = MASTERHOUSE->query_house(invitee);
   if (!house && permanent && (MASTERHOUSE->query_house_exists(invitee))) {
      house = clone_object(HOUSE);
      house->set_owner(invitee);
      house->restore_stats(invitee);
   }
   return house;
}

