
 /* House_item_editor.c      Metaprat       3rd August 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


static string items;
static string item_selected;


/* standard functions */


id(str) {
   return (str == "house item editor") || (str == "item editor") ||
          (str == "editor");
}


short() {
   return "house item editor";
}


long() {
   write("A house item editor.\n");
   write("It allows the owner of a house to modify the 'items' (examinable\n");
   write("objects) defined in the current room.  Commands available:\n");
   write("\titems   \t- Puts you into the item editor menu.\n");
   write("Once you are there, you are offered the following choices:\n");
   write("\tq       \t- quit items menu, loosing new items list\n");
   write("\tu       \t- update the room with the new items list\n");
   write("\ta <name>\t- add item from the list\n");
   write("\td <name>\t- delete item from the list\n");
   write("\tl       \t- list items\n");
   write("\th or ?  \t- help\n");
   write("The format of the items list  is:\n");
   write("  <name> -> <description>   repeated up to 10 times.  If you\n");
   write("  'examine <name>' or 'look at <name>', you see <description>.\n");
   write("When adding an item, you will be prompted for the <desc>, which\n");
   write("is a single line of text, to which a '.' is added automatically.\n");
}


toolkit_init() {
   add_action("items", "items");
}


write_items_menu() {
   write("Items menu:\n");
   write(" q        - quit items menu, loosing new items list\n");
   write(" u        - update the room with the new items list\n");
   write(" a <name> - add item\n");
   write(" d <name> - delete item\n");
   write(" l        - list items\n");
   write(" h or ?   - help\n");
}


items(str) {
   object this_main_hall;

   this_main_hall = environment(this_player())->query_main_hall();
   if (!this_main_hall) {
      write("You are not in a house.\n");
      return 1;
   }
   if (this_main_hall != main_hall) {
      write("You are not in your own house.\n");
      return 1;
   }
   this_room_index = environment(this_player())->query_room_index();
   items = main_hall->get_rooms_items(this_room_index);
   if (!items) {
      items = ({ });
   }
   write_items_menu();
   list_items();
   input_to("edit_items");
   return 1;
}



set_items_info() {
   object here;

   MONEY->charge_money(ITEMCOST, "items");
   write("You update " + main_hall->get_rooms_short(this_room_index) + ".\n");
   main_hall->set_rooms_items(this_room_index, items);
   main_hall->save_stats();
   this_room_index = -1;
   items = ({ });
   return 1;
}


edit_items(str) {
   if (!MONEY->has_money(ITEMCOST)) {
      return 1;
   }
   if (!str || (str == "")) {
      write_items_menu();
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   if (lower_case(str) == "q") {
      write("Leaving items menu, discarding new items list.\n");
      this_room_index = -1;
      items = ({ });
      return 1;
   }
   if (lower_case(str) == "a") {
      write("Please enter the name of the item to add : ");
      input_to("select_add_item");
      return 1;
   }
   if (sscanf(lower_case(str), "a %s", item_selected) == 1) {
      write("Please enter description text.  Use up to 70 characters,\n");
      write("punctuation will be added automatically.\n");
      write(">> ");
      input_to("add_item");
      return 1;
   }
   if (lower_case(str) == "d") {
      write("Please enter the name of the item to delete : ");
      input_to("select_delete_item");
      return 1;
   }
   if (sscanf(lower_case(str), "d %s", item_selected) == 1) {
      delete_item();
      return 1;
   }
   if (lower_case(str) == "l") {
      list_items();
      input_to("edit_items");
      return 1;
   }
   if (lower_case(str) == "u") {
      set_items_info();
      return 1;
   }
   if ((lower_case(str) == "h") || (str == "?")) {
      write("Items consist of an identifier, followed by a single line of\n");
      write("description.  Eg,\n");
      write(" \"kitchen sink\" & \"A sink, full of unwashed dishes\"\n");
      write("Punctuation is added on later by the examine/look at command.\n");
      write("The items menu allows you to add and remove items from the\n");
      write("items list, so for example, to add the above item,\n");
      write(" Select: a kitchen sink          (add a new item to the list)\n");
      write(" Please enter a line of text.\n");
      write(" > A sink, full of unwashed dishes\n");
      write(" Select: l                       (list items in list)       \n");
      write(" [1] \"kitchen sink\" -> \"A sink, full of unwashed dishes.\"\n");
      write(" Select: d kitchen sink          (delete the entry)         \n");
      write(" Item deleted.                                              \n");
      write(" Select: l                       (list items in list)       \n");
      write(" No items exist.                                            \n");
      write(" Select: q                       (leave items system)       \n");
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   write("Didn't understand item menu selection.\n");
   write_items_menu();
   write("Select: ");
   input_to("edit_items");
   return 1;
}



select_add_item(str) {
   if (str && (str != "")) {
      item_selected = str;
      write("Please enter description text.  Use up to 70 characters,\n");
      write("punctuation will be added automatically.\n");
      write(">> ");
      input_to("add_item");
   }
   else {
      write("No item name provided.  Returning to items menu.\n");
      write("Select: ");
      input_to("edit_items");
   }
}



select_delete_item(str) {
   if (str && (str != "")) {
      item_selected = str;
      delete_item();
   }
   else {
      write("No item name provided.  Returning to items menu.\n");
      write("Select: ");
      input_to("edit_items");
   }
}



add_item(str) {
   if (!str || (str == "")) {
      write("No description supplied,  returning to items menu.\n");
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   if (strlen(str) > MAXITEMLEN) {
      write("The description exceeds the maximum item size.  Ignored.\n");
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   if (sizeof(items) >= (MAXITEMNUM * 2)) {
      write("You can't add any more items, since the limit is reached.\n");
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   if (search_paired_array(items, item_selected)) {
      items = remove_paired_array_item(items, item_selected);
   }
   items = items + ({ item_selected, str });
   write("Select: ");
   input_to("edit_items");
   return 1;
}



delete_item() {
   int tmp_size;

   if (sizeof(items) <= 0) {
      write("No items in list.\n");
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   tmp_size = sizeof(items);
   items = remove_paired_array_item(items, item_selected);
   if (tmp_size == sizeof(items)) {
      write("Nothing removed from items list.\n");
   }
   else {
      write("" + ((tmp_size - sizeof(items)) / 2) + " item removed.\n");
   }
   write("Select: ");
   input_to("edit_items");
   return 1;
}



list_items() {
   int counter;

   if (sizeof(items) <= 0) {
      write("No items in list.\n");
      write("Select: ");
      input_to("edit_items");
      return 1;
   }
   for (counter = 0; counter < sizeof(items) - 1; counter += 2) {
      write("[" + (counter / 2 + 1)  + "]\t\"" + items[counter] +
            "\"\t->\t\"" + items[counter + 1] + "\".\n");
   }
   write("Select: ");
   input_to("edit_items");
   return 1;
}



