
 /* house_map.c     Metaprat    11th April 1992  */

 /* builds up a map of the house */


#include "house_defines.h"

/* STATIC variables (won't be restored into) */
static string index_desc;
static string map_desc;
static string owner_name;
static object main_hall;


/* DYNAMIC variables (will be restored) */
string rooms_short;
int    rooms_x;
int    rooms_y;
int    rooms_z;
int    exits_north;
int    exits_ne;
int    exits_east;
int    exits_se;


id(str) {
   return (str == "map")   ||   (str == owner_name + " house map")  ||
          (str == short()) ||   (str == owner_name + " map")        ||
          (str == "map of " + owner_name + "'s house")  ||
          (str == "map of " + owner_name + "s house")   ||
          (str == "index")  ||  (str == "map index");
}


short() {
   if (!owner_name) {
      return "Undrawn map";
   }
   return "Map of " + capitalize(owner_name) + "'s house, with index";
}


get() {
   return 1;
}


drop() {
   return 0;
}



query_value() {
   return (MAPCOST / 2) ;
}


reset(arg) {
   if (arg) {
      return;
   }
   if (DEBUGLEVEL > 2) {
      log_file(LOGFILE, "House_map: created.\n");
   }
}



init() {
   add_action("read", "read");
   if (IAMWIZ) {
      add_action("draw_house_of", "draw_house_of");
   }
}



draw_map() {
   string map_row_1;
   string map_row_2;
   int    index_room;
   int    index_z;
   int    min_z;
   int    max_z;
   int    range_z;

   if (DEBUGLEVEL) {
      log_file(LOGFILE, "House_map: drawing map, ");
      if (this_player()) {
         log_file(LOGFILE, "for " + MYCNAME + ", ");
      }
      else {
         log_file(LOGFILE, "No player name known.\n");
      }
      log_file(LOGFILE, "house owner is " + owner_name + ".\n");
   }
   if (!restore_object(HOUSES_DIR + owner_name)) {
      map_desc = "The map is an empty page.\n";
      index_desc = "The index page is blank.\n";
      return 0;
   }
   if (!exits_se) { exits_se = allocate(sizeof(rooms_short)); }
   if (!exits_ne) { exits_ne = allocate(sizeof(rooms_short)); }
   if (!rooms_z)  { rooms_z  = allocate(sizeof(rooms_short)); }
   /* Calculate page size */
   for (index_room = 0; index_room < sizeof(rooms_z); index_room ++) {
      if (rooms_short[index_room]) {
         if (rooms_z[index_room] > max_z) { max_z = rooms_z[index_room]; }
         if (rooms_z[index_room] < min_z) { min_z = rooms_z[index_room]; }
      }
   }
   range_z = max_z - min_z + 1;
   if (IAMWIZ) {
      write(" Map size: " + min_z + " <= Z <= " + max_z + " floors.\n");
   }
   map_desc = "A map of " + capitalize(owner_name) + "'s house.\n";
   for (index_z = min_z; index_z <= max_z; index_z ++) {
      int    index_x;
      int    index_y;
      int    min_x;
      int    max_x;
      int    min_y;
      int    max_y;
      int    range_x;
      int    range_y;

      max_x = -999;  /* make sure the maxes & mins get reset */
      min_x = 999;
      max_y = -999;
      min_y = 999;
      for (index_room = 0; index_room < sizeof(rooms_x); index_room ++) {
         if (rooms_short[index_room] && (rooms_z[index_room] == index_z)) {
            if (rooms_x[index_room] > max_x) { max_x = rooms_x[index_room]; }
            if (rooms_x[index_room] < min_x) { min_x = rooms_x[index_room]; }
            if (rooms_y[index_room] > max_y) { max_y = rooms_y[index_room]; }
            if (rooms_y[index_room] < min_y) { min_y = rooms_y[index_room]; }
         }
      }
      range_x = max_x - min_x + 1;
      range_y = max_y - min_y + 1;
      if ((range_x <= 0) || (range_y <= 0)) {
         /* no rooms on this floor */
         continue;
      }
      if (IAMWIZ) {
         write("(floor " + index_z + ":, " +
               min_x + " <= X <= " + max_x + ", " +
               min_y + " <= Y <= " + max_y + ")\n");
      }
      range_x = max_x - min_x + 1;
      range_y = max_y - min_y + 1;
      /* set up the room matrix */
      map_row_2 = allocate(range_y);
      map_row_1 = allocate(range_y);
      for (index_y = 0; index_y < range_y; index_y ++) {
         map_row_2[index_y] = allocate(range_x * 2);
         map_row_1[index_y] = allocate(range_x * 2);
         for (index_x = 0; index_x < (range_x * 2); index_x ++) {
            map_row_2[index_y][index_x] = " ";
            map_row_1[index_y][index_x] = " ";
         }
      }
      for (index_room = 0; index_room < sizeof(rooms_x); index_room ++) {
         if (rooms_short[index_room] && (rooms_z[index_room] == index_z)) {
            index_x = rooms_x[index_room] - min_x;
            index_y = rooms_y[index_room] - min_y;
            map_row_2[index_y][index_x * 2]       =
                     ({"E","1","2","3","4","5","6","7","8","9",
                       "a","b","c","d","e","f","g","h","i","j",
                       "k","l","m","n","o","p","q","r","s","t",
                       "u","v","w","x","y","z","A","B","C","D",
                       "F","G","H","J","K","L","M","N","O","P"})[index_room];
            if ((exits_north[index_room]) &&
                (exits_north[index_room] != DOOR_HIDDEN)) {
               map_row_1[index_y][index_x * 2] = "|";
            }
            if ((exits_ne[index_room]) &&
                (exits_ne[index_room] != DOOR_HIDDEN)) {
               if (map_row_1[index_y][index_x * 2 + 1] == "\\") {
                  map_row_1[index_y][index_x * 2 + 1] = "X";
               }
               else {
                  map_row_1[index_y][index_x * 2 + 1] = "/";
               }
            }
            if ((exits_east[index_room]) &&
                (exits_east[index_room] != DOOR_HIDDEN)) {
               map_row_2[index_y][index_x * 2 + 1] = "-";
            }
            if ((exits_se[index_room]) &&
                (exits_se[index_room] != DOOR_HIDDEN)) {
               if (map_row_1[index_y + 1][index_x * 2 + 1] == "/") {
                  map_row_1[index_y + 1][index_x * 2 + 1] = "X";
               }
               else {
                  map_row_1[index_y + 1][index_x * 2 + 1] = "\\";
               }
            }
         }
      }
      if (range_z > 1) {
         map_desc += "(Floor " + index_z + "):\n";
      }
      for (index_y = 0; index_y < range_y; index_y ++) {
         map_desc += "    " + implode(map_row_1[index_y], "") + "\n" +
                     "    " + implode(map_row_2[index_y], "") + "\n";
      }
      map_row_1 = 0;
      map_row_2 = 0;
   }
   /* produce the index page */
   if (rooms_short[0]) {
      index_desc = "E - Entrance/Exit (" + rooms_short[0] + ").";
   }
   else {
      index_desc = "E - Entrance/Exit.";
   }
   for (index_room = 1; index_room < sizeof(rooms_x); index_room ++) {
      if (rooms_short[index_room]) {
         index_desc = index_desc + "\n" +
                  ({"E","1","2","3","4","5","6","7","8","9",
                    "a","b","c","d","e","f","g","h","i","j",
                    "k","l","m","n","o","p","q","r","s","t",
                    "u","v","w","x","y","z","A","B","C","D",
                    "F","G","H","J","K","L","M","N","O","P"})[index_room] +
            " - " + rooms_short[index_room] + ".";
      }
   }
   rooms_short = ({ });  /* clear it out */
   return 1;
}



draw_house_of(str) {
   if (!str) {
      write("Usage:   draw_house_of <name>\n");
      return 1;
   }
   if (set_owner(lower_case(str))) {
      write("Map drawn successfully for " + capitalize(str) + ".\n");
   }
   else {
      write("No such person found.\n");
   }
   return 1;
}


set_owner(str) {
   owner_name = str;
   call_out("timeout", TIMEOUT);
   return draw_map();
}

read(str) {
   if ((!str) || (!id(str))) {
      return 0;
   }
   if ((str == "index") || (str == "map index")) {
      write(index_desc + "\n");
      return 1;
   }
   write(map_desc + "\nE - Entrance/Exit.\nThe index may also be read.\n");
   return 1;
}


long(str) {
   if ((str == "index") || (str == "map index")) {
      write(index_desc + "\n");
      return 1;
   }
   write(map_desc + "\nE - Entrance/Exit.\nThe index may also be read.\n");
   return 1;
}


timeout() {
   say("Your map turns to dust.\n");
   map_desc   = ""; /* just make sure the arrays get deallocated */
   index_desc = "";
   destruct(this_object());
}



