
 /* insurancequote.c      Metaprat     1st April 1992  */

 /* functions:                                         */
 /*    set_value()      -  sets amount of compensation */
 /*    set_cust_name()  -  real name of customer       */
 /*    set_locker()     -  filename of storage object  */
 /*    set_id()         -  random key for policy       */
 /*    query_amount()   -  compensation                */
 /*    query_cust_name()-  name of customer            */
 /*    query_locker()   -  filename                    */
 /*    query_id_key()   -  returns random key          */
 /*    parent_locker()  -  object ptr back to locker   */

#define DEBUGON  1
#define FILENAME "/players/metaprat/misc_objs/insurancequote"
#define LOGFILE  "metaprat.lockers"

int    policy_taken;
int    price;
int    amount;
string cust_name;
string container;
string id_key;
object parent_ob;


id(str) {
   if (policy_taken) {
      return (str == "locker policy")    ||    (str == id_key)     ||
             (str == "insurance policy") ||    (str == "policy");
   }
   return (str == "locker quote")    ||    (str == id_key)     ||
          (str == "insurance quote") ||    (str == "quote");
}


short() {
   if (policy_taken) {
      return "insurance policy for " + amount + " gp";
   }
   return "insurance quote";
}


long() {
   if (policy_taken) {
      write("+-------------------------------------------------------+\n");
      write("|ACME Insurance PLC                    Policy number:  \t|\n");
      write("|                                      "+id_key+"      \t|\n");
      write("| On behalf of 'Secure Lockers Ltd'.                   \t|\n");
      write("| To claim, see the attendant and      Coverage:       \t|\n");
      write("| 'claim'.                             "+amount+" gp     \t|\n");
      write("+-------------------------------------------------------+\n");
      return 1;
   }
   if (amount < price) {
      write("Invalid insurance quote.  They're trying to make you pay\n");
      write("more money for the insurance policy than your stuff is\n");
      write("actually worth.\n");
      return 1;
   }
   write("Insurance quote for " + amount + " gold coins.\n");
   write("You must type 'accept quote' within a minute to take the\n");
   write("policy, which will cost you " + price + " gp.\n");
   return 1;
}


get() {
   return 0;     /* may not be picked up */
}


drop() {
   return 1;     /* may not be dropped */
}


query_weight() {
   return 0;     /* weightless  */
}


query_value() {
   return 0;     /* may not be sold */
}


query_auto_load() {
   if (policy_taken) {
      return FILENAME + ":" + amount + ";" +
                              cust_name + ";" +
                              container + ";" +
                              id_key;
   }
   return 0;
}


init_arg(arg) {
   sscanf(arg,"%d;%s;%s;%s",amount,cust_name,container,id_key);
   policy_taken = 1;
}


init() {
   if (!policy_taken) {
      add_action("accept_quote","accept");
   }
   if (this_player()->query_level() > 20) {
      add_action("policy_stats","policy_stats");
      add_action("policy_stats","quote_stats");
   }
}


accept_quote(str) {
   if ((str) && (!id(str))) {
      return 0;
   }
   if (price > amount) {
      write("Do you want to lose money or something?? Forget it!\n");
      return 1;
   }
   if (this_player()->query_money() < price) {
      write("You do not have sufficient funds to buy the policy.\n");
      return 1;
   }
   if (DEBUGON) {
      log_file(LOGFILE,
         this_player()->query_name() + " buys policy, " +
         "Price: " + price + " gp, " +
         "Coverage: " + amount + " gp, " +
         "Container: " + container + ",\n" +
         "at time: " + ctime(time()) + ", " +
         "Policy id: " + id_key + ".\n");
   }
   if (parent_ob) {
      parent_ob->add_revenue(price);
   }
   this_player()->add_money(-price);
   say(this_player()->query_name() + " decides to take out an insurance " +
       "policy.\n",
       this_player());
   write("You take out an insurance policy.\n");
   policy_taken = 1;
   move_object(this_object(),environment(this_object()));
   remove_call_out("timeout");
   return 1;
}



policy_stats() {
   write("Policy statistics:\n");
   write("Amount of policy: " + amount + " gp.\n");
   write("Customer name:    " + cust_name + ".\n");
   write("Container:        " + container + ".\n");
   write("Policy ID:        " + id_key    + ".\n");
   if (policy_taken) {
      write("Policy paid up.\n");
   }
   else {
      write("Policy not yet paid.\n");
   }
   if (parent_ob) {
      write("Locker:           " + file_name(parent_ob) + ".\n");
   }
   else {
      write("Locker:           No pointer to locker object.\n");
   }
   return 1;
}


set_value(arg) {
   if (arg > 0) {
      amount = arg;
      price = 10 + (amount / 20);
   }
   else {
      amount = 0;
      price = 10;
   }
   call_out("timeout",60);
}


set_cust_name(str) {
   cust_name = str;
}

set_locker(str) {
   container = str;
}

set_id(str) {
   id_key = str;
}

query_amount() {
   return amount;
}

query_cust_name() {
   return cust_name;
}

query_locker() {
   return container;
}

query_id_key() {
   return id_key;
}


timeout() {
   if (policy_taken) {
      return;
   }
   say("The insurance quote turns into dust.\n");
   destruct(this_object());
}


parent_locker(arg) {
   parent_ob = arg;
}

