

 /* lockers.c      Metaprat       31st March 1992      */

 /* Lockers, that the player can put stuff in for       */
 /* short periods.  He/she is given a key (autoloading) */
 /* after paying a deposit.  Only one locker may be     */
 /* used by the player at a time.                       */
 /* In addition, 'insurance policies' may be taken out  */
 /* for part of the value of the contents. NOTE: these  */
 /* will ALWAYS be less than the amount the shop would  */
 /* give out.  The insurance policy pays out if the mud */
 /* shuts down or crashes while goods are still in the  */
 /* locker.                                             */


#define STORAGEOBJ      "/obj/bag"     /* anything will do really */
#define INSURQUOTE      "/players/metaprat/misc_objs/insurancequote"
#define LOGFILE         "metaprat.policy"
#define NUMBERLOCKERS   10
#define LOCKERCHARGE    50
#define TIMEOUT         30             /* secs before throws stuff out */
#define NOBODY          "XXnobodyXX"


int    revenue_so_far;
int    current_locker;
string customer_name;
int    customer_time;
int    hiring_locker;

string locker_id_key;      /* arrays to hold info about individual */
object locker_policy;      /* lockers  (random key for ID, ptr to  */
object locker_storage;     /* customers policy/quote, ptr to the   */
string locker_owner;       /* storage object, and his/her name).   */


init() {
   add_action("put",   "put");      /* put all in locker  command */
   add_action("hire",  "hire");     /* get a new locker    */
   add_action("lock",  "lock");     /* lock it, pay charge */
   add_action("lock",  "close");
   add_action("claim", "claim");    /* get insurance pay-off */
   add_action("unlock","unlock");   /* unlock your locker  */
   add_action("unlock","open");
   add_action("ask",  "ask");       /* get information */
   if (this_player()->query_level() > 20) {
      add_action("locker_stats","locker_stats");
   }
}


id(str) {
   return (str == "lockers") || (str == "locker") ||
          (str == "locker attendant") || (str == "attendant");
}


short() {
   return "Row of secure lockers, and an attendant";
}


long(str) {
   int whichlocker;

   if ((str == "locker attendant") || (str == "attendant")) {
      write("The locker attendant is a surly thug, who is not above\n");
      write("pilfering the occasional item left to his 'care'.. When\n");
      write("he hears the words 'politeness', or 'customer satisfaction'\n");
      write("he reaches for the dictionary.. to throw it away.\n");
      return 1;
   }
   write("This is a bank of lockers, all controlled by an attendant.  He\n");
   write("will only deal with one person at a time, but will get impatient\n");
   write("very quickly if that person does not fill or empty his locker\n");
   write("immediately.  On the positive side, you may buy contents\n");
   write("insurance for your stuff, in case the mud crashes or shuts down.\n");
   write("To find out more,    'ask attendant about lockers'\n");
   write("                     ' ..     ..      ..  insurance'\n");
   whichlocker = searchlockers(this_player()->query_real_name());
   if (whichlocker > 0) {
      write("You notice that locker number " + whichlocker + " has your " +
            "name written\non the front.  Spelt wrongly.\n");
   }
   else {
      if (searchlockers(NOBODY) > 0) {
         write("Some of the lockers seem to be empty.\n");
      }
      else {
         write("All of the lockers seem to be in use.\n");
      }
   }
   if (current_locker > 0) {
      write("The attendant has opened locker number " + current_locker +
            ", which seems\nto belong to " + capitalize(customer_name) +
            ".\n");
   }
   return 1;
}


ask(str) {
   string subject;

   if (!str) {
      return 0;
   }

   if (sscanf(str,"attendant about %s",subject) != 1) {
      if (sscanf(str,"locker attendant about %s",subject) != 1) {
         return 0;
      }
   }
   if ((subject != "lockers") && (subject != "insurance")) {
      tell_room(environment(this_object()),
          this_player()->query_name() + " asks the Locker attendant " +
          "something.\n");
      tell_room(environment(this_object()),
          "Locker attendant says: You tryin' to confuse me or sumfin " +
          this_player()->query_name() + "?\n");
      tell_room(environment(this_object()),
          "Locker attendant says: all I have to tell you about is " +
          "lockers and insurance.. anything\n");
      tell_room(environment(this_object()),
          "Locker attendant says: else and you can fuck right off!\n");
      return 1;
   }
   if (subject == "lockers") {
      say("Locker attendant says something to " +
          this_player()->query_name() + ".\n",
          this_player());
      write("Erm, well like, you can hire one locker, and one locker only,\n");
      write("at a time, or I'll fuckin knack ya.  It costs " +
            LOCKERCHARGE + " gold.\n");
      write("You'd better be fuckin quick about putting yer rubbish into\n");
      write("the locker, or I'll take ya money anyway for wastin me time.\n");
      write("The stuff'll be safe in there unless the game shuts down, in\n");
      write("which case you shoulda bought a fuckin insurance policy...\n");
      write("If you wanna locker,  'hire locker', an I'll gi ya one if\n");
      write("I got one free.  'put all in locker',  then 'lock locker'.\n");
      write("I'll gi ya an insurance quote, but you gorra decide to tak it\n");
      write("right quick.  If ya wanna know more about insurance, ask me.\n");
      write("Ta git ya rubbish back, 'unlock/open locker', ya can either\n");
      write("empty it out or lock it again.\n");
      return 1;
   }
   if (subject == "insurance") {
      say("Locker attendant says something to " +
          this_player()->query_name() + ".\n",
          this_player());
      write("I'll gi ya an insurance quote when ya ask me ta lock ya\n");
      write("locker.  It costs: 10 + (1/20th) the cost of the rubbish\n");
      write("ya chucked in yer locker.  If ya stuff gets knacked, like\n");
      write("the mud goes down or summat, ya get back 95% of wot it was\n");
      write("worth, but I aint paying back more than 1000 gold on any\n");
      write("single item.  Oh yeah, you gorra pay the first 50 gold on\n");
      write("the total anyway, like.\n");
      write("If ya wanna claim on a policy, come here and 'claim'\n");
      return 1;
   }
}


query_value() {
   return 0;
}


query_weight() {
   return 0;
}


get() {
   return 0;
}


can_put_and_get() {
   if (current_locker == 0) {
      return 0;
   }
   if (this_player()->query_real_name() != customer_name) {
      tell_room(environment(this_object()),
          "Locker attendant pushes " + this_player()->query_name() +
          " away from the lockers.\n");
      tell_room(environment(this_object()),
          "Locker attendant says: Git ya theivin hands off that locker!\n");
      tell_room(environment(this_object()),
          "Locker attendant says: or I'll fuckin knack ya! Thats " +
          capitalize(customer_name) + "'s fuckin locker.\n");
     return 0;
   }
   return 1;
}


add_weight(w) {
   if ((!current_locker) ||
       (this_player()->query_real_name() != customer_name)) {
      return 0;
   }
   return 1;
}


lock(str) {
   int    value;
   object quote;

   if ((!str) || (!id(str))) {
      return 0;
   }
   if (!current_locker) {
      write("Locker attendant says: Ya have to fuckin HIRE it first!\n");
      return 1;
   }
   if (this_player()->query_real_name() != customer_name) {
      tell_room(environment(this_object()),
          "Locker attendant says: Oy " + this_player()->query_name() +
          "! Thats " + capitalize(customer_name) + "'s\n" +
          "fukkin locker, not yours..\n");
      return 1;
   }
   if (this_player()->query_money() < LOCKERCHARGE) {
      tell_room(environment(this_object()),
          "Locker attendant growls, and says: " +
          this_player()->query_name() + ", you're fukkin skint!\n");
      tell_room(environment(this_object()),
          "Locker attendant says: The fukka costs " +
          LOCKERCHARGE + " gold coins.\n");
      tell_room(environment(this_object()),
          "Locker attendant says: take your junk and stop wasting my time!\n");
      tell_room(environment(this_object()),
          "Locker attendant throws everything onto the floor.\n");
      move_contents(this_object(),environment(this_object()));
      return 1;
   }
   write("Locker attendant takes " + LOCKERCHARGE + " gp from you.\n");
   tell_room(environment(this_object()),
       "Locker attendant slams the locker shut, and locks it.\n");
   say("Locker attendant gives " + capitalize(customer_name) +
       " an insurance quote.\n",
       this_player());
   write("Locker attendant gives you an insurance quote, and says: Ya gorra\n"+
         "minute to make up ya mind, or that quote's invalid.\n");
   this_player()->add_money(-LOCKERCHARGE);
   revenue_so_far += LOCKERCHARGE;
   locker_owner[current_locker] = this_player()->query_real_name();
   locker_id_key[current_locker] = build_random_key();
   move_contents(this_object(),locker_storage[current_locker]);
   value = value_contents(locker_storage[current_locker]);
   quote = clone_object(INSURQUOTE);
   quote->parent_locker(this_object());
   quote->set_value(value);
   quote->set_cust_name(this_player()->query_real_name());
   quote->set_locker(file_name(locker_storage[current_locker]));
   quote->set_id(locker_id_key[current_locker]);
   locker_policy[current_locker] = quote;
   move_object(quote,this_player());
   remove_call_out("timeout");
   customer_name = NOBODY;
   current_locker = 0;
   hiring_locker = 0;
   return 1;
}


unlock(str) {
   object currentpolicy;
   object thisob;
   int whichone;

   if (!id(str)) {
      return 0;
   }
   say(this_player()->query_name() + " asks the attendant something.\n",
       this_player());
   if (current_locker > 0) {
      if (this_player()->query_real_name() != customer_name) {
         tell_room(environment(this_object()),
             "Locker attendant gobs on " +
             this_player()->query_real_name() + ".\n");
         tell_room(environment(this_object()),
             "Locker attendant says: fuck off, I'm talking to " +
            capitalize(customer_name) + ".\n");
         return 1;
      }
      else {
         write("Locker attendant says: fuck off! You haven't even _locked_\n" +
               "it yet...\n");
         return 1;
      }
   }
   tell_room(environment(this_object()),
       "Locker attendant stares at " + this_player()->query_name() +
       ", and then looks at the rack of lockers.\n");
   whichone = searchlockers(this_player()->query_real_name());
   if (whichone == 0) {
      tell_room(environment(this_object()),
          "Locker attendant picks his nose, then says: Nope, ain't gorra\n");
      tell_room(environment(this_object()),
          "fuckin locker with your name on it.  If ya lost ya stuff, ya'd\n");
      tell_room(environment(this_object()),
          "berra have a fukkin insurance policy, or ya knacked..\n");
      return 1;
   }
   tell_room(environment(this_object()),
       "Locker attendant unlocks one of the lockers.\n");
   write("Locker attendant says: Heres your fukkin stuff, now tak it or\n");
   write("lock it again..\n");
   current_locker = whichone;
   hiring_locker = 0;
   thisob = first_inventory(this_player());
   currentpolicy = locker_policy[current_locker];
   if (!currentpolicy) {
      while (thisob) {
         if ((thisob->id("locker policy")) &&
             (thisob->query_id_key() == locker_id_key[current_locker])) {
            currentpolicy = thisob;
         }
         thisob = next_inventory(thisob);
      }
   }
   if (currentpolicy) {
      write("He then takes the old policy from your pocket.\n");
      destruct(currentpolicy);
   }
   customer_name = this_player()->query_real_name();
   customer_time = time();
   move_contents(locker_storage[current_locker],this_object());
   locker_owner[current_locker]   = NOBODY;
   locker_id_key[current_locker]  = "cleared";
   call_out("timeout",TIMEOUT);
   return 1;
}


hire(str) {
   int whichone;

   if (!id(str)) {
      return 0;
   }
   say(this_player()->query_name() + " tries to hire a locker.\n",
       this_player());
   if (current_locker > 0) {
      if (this_player()->query_real_name() != customer_name) {
         tell_room(environment(this_object()),
             "Locker attendant gobs on " +
             this_player()->query_real_name() + ".\n");
         tell_room(environment(this_object()),
             "Locker attendant says: fuck off, I'm talking to " +
             capitalize(customer_name) + ".\n");
         return 1;
      }
      else {
         write("Locker attendant says: fuck off! You haven't even _locked_\n" +
               "it yet...\n");
         return 1;
      }
   }
   tell_room(environment(this_object()),
       "Locker attendant stares at " + this_player()->query_name() +
       ", and then looks at the rack of lockers.\n");
   whichone = searchlockers(this_player()->query_real_name());
   if (whichone > 0) {
      tell_room(environment(this_object()),
          "Locker attendant screams: You already got a blasted locker!\n");
      tell_room(environment(this_object()),
          "Unlock it, don't hire another one! fukkin cunt..\n");
      return 1;
   }
   whichone = searchlockers(NOBODY);
   if (whichone == 0) {
      tell_room(environment(this_object()),
          "Locker attendant says: Ya blind?? They're all full..\n");
      return 1;
   }
   tell_room(environment(this_object()),
       "Locker attendant opens an empty locker.\n");
   write("Locker attendant says: Heres ya go, get ya junk in there and\n");
   write("lock it, quick.\n");
   current_locker = whichone;
   hiring_locker = 1;
   customer_name = this_player()->query_real_name();
   customer_time = time();
   call_out("timeout",TIMEOUT * 2);
   return 1;
}



timeout() {
   object nextob;

   nextob = first_inventory(this_object());
   if (nextob) {
      hiring_locker = 0;
      if (!find_player(customer_name)) {
         tell_room(environment(this_object()),
             "Locker attendant says: Where the fuck is " +
             capitalize(customer_name) + "??\n" +
             "Well this crap is going out on the floor!\n");
         move_contents(this_object(),environment(this_object()));
         customer_name = NOBODY;
         current_locker = 0;
         tell_room(environment(this_object()),
             "Locker attendant says: Next customer, get your arse up here!\n");
         return;
      }
      tell_room(environment(this_object()),
          "Locker attendant shouts: Right!  You've had more than enough\n" +
          "time, " + capitalize(customer_name) + ", this lots going out\n" +
          "on the floor!\n");
      tell_room(environment(this_object()),
          "Locker attendant grabs the contents of the locker, and flings\n" +
          "it all over the room.\n");
      move_contents(this_object(),environment(this_object()));
      say("Locker attendant says: And there's a " + LOCKERCHARGE + " gp " +
          "fine for wasting\nmy time as well!\n",
          find_player(customer_name));
      find_player(customer_name)->add_money(-LOCKERCHARGE);
      revenue_so_far += LOCKERCHARGE;
      tell_object(find_player(customer_name),
         "Locker attendant fines you for wasting time.\n");
      customer_name = NOBODY;
      current_locker = 0;
      tell_room(environment(this_object()),
          "Locker attendant says: Next! Get a move on, this isn't a " +
          "charity.\n");
      return;
   }
   if (hiring_locker) {
      tell_room(environment(this_object()),
          "Locker attendant shouts: What the fuck, " +
          capitalize(customer_name) + " was\n" +
          "supposed to be _hiring_ this friggin locker..\n");
      if (find_player(customer_name)) {
         tell_object(find_player(customer_name),
            "Locker attendant fines you for wasting time.\n");
         find_player(customer_name)->add_money(-LOCKERCHARGE);
         revenue_so_far += LOCKERCHARGE;
      }
      customer_name = NOBODY;
      current_locker = 0;
      hiring_locker = 0;
      return;
   }
   tell_room(environment(this_object()),
       "Locker attendant says: You finished with your locker, " +
       capitalize(customer_name) + ".\nNext sucker please!\n");
   customer_name = NOBODY;
   current_locker = 0;
}


move_contents(origin, destination) {
   object thisob;
   object nextob;

   thisob = first_inventory(origin);
   while (thisob) {
      nextob = next_inventory(thisob);
      move_object(thisob,destination);
      thisob = nextob;
   }
}


value_contents(container) {
   object thisob;
   int    totval;

   totval = 0;
   thisob = first_inventory(container);
   while (thisob) {
      if (thisob->query_value() > 1000) {
         totval += 1000;
      }
      else {
         totval += thisob->query_value();
      }
      thisob = next_inventory(thisob);
   }
   return (totval * 95) / 100 - 50;
}


build_random_key() {
   int    indx;
   string key;

   key = "";
   for (indx = 1; indx < 10; indx ++) {
      key = key + random(10);
   }
   return key;
}


reset(arg) {
   int indx;

   if (arg)
      return;
   revenue_so_far = 0;
   current_locker = 0;
   customer_name  = NOBODY;
   customer_time  = 0;

   locker_storage = allocate(NUMBERLOCKERS + 1);
   locker_owner   = allocate(NUMBERLOCKERS + 1);
   locker_id_key  = allocate(NUMBERLOCKERS + 1);
   locker_policy  = allocate(NUMBERLOCKERS + 1);

   /* NOTE: locker 0 is a dummy */
   for (indx = 1; indx <= NUMBERLOCKERS; indx++) {
      locker_storage[indx] = clone_object(STORAGEOBJ);
      locker_owner[indx]   = NOBODY;
      locker_id_key[indx]  = "never set";
   }
}



locker_stats() {
   int indx;

   if (this_player()->query_level() < 20) {
      return;
   }
   write("Locker statistics:\n");
   write("File name:      " + file_name(this_object())    + ".\n");
   write("Storage objs:   " + STORAGEOBJ                  + ".\n");
   write("Insurance quot: " + INSURQUOTE                  + ".\n");
   write("Max num locker: " + NUMBERLOCKERS               + ".\n");
   write("Hire charge:    " + LOCKERCHARGE             + " gp.\n");
   write("Timeout period: " + TIMEOUT                + " secs.\n");
   write("Revenue so far: " + revenue_so_far              + ".\n");
   write("Current locker: " + current_locker              + ".\n");
   write("Customer name:  " + customer_name               + ".\n");
   write("Time of start:  " + ctime(customer_time)        + ".\n");
   write("Time now:       " + ctime(time())               + ".\n");
   for (indx = 1; indx <= NUMBERLOCKERS; indx ++) {
      write("Locker " + indx + ":\t" + locker_owner[indx] + ",\t");
      write(file_name(locker_storage[indx])               + ",\n");
      write("(key:) " + locker_id_key[indx]               + ",\t");
      if (locker_policy[indx]) {
         write(file_name(locker_policy[indx])             + ".\n");
      }
      else {
         write("No policy object"                         + ".\n");
      }
   }
   return 1;
}


searchlockers(str) {
   int indx;

   for (indx = 1; indx <= NUMBERLOCKERS; indx ++) {
      if (str == locker_owner[indx]) {
         return indx;
      }
   }
   return 0;
}


claim() {
   object policy;
   object thisob;
   object currentpolicy;
   object bag;
   int    whichlocker;
   int    indx;
   int    value;

   log_file(LOGFILE,
            "Policy claim!  By " +
            "name: "      + this_player()->query_real_name() + ", " +
            "time now: "  + ctime(time()) + ",\n");
   tell_room(environment(this_object()),
       this_player()->query_name() + " gives the Locker attendant an " +
       "insurance claim.\n");
   whichlocker = searchlockers(this_player()->query_real_name());
   thisob = first_inventory(this_player());
   while (thisob) {
      if (thisob->id("locker policy")) {
         if ((whichlocker) &&
             (thisob->query_id_key() == locker_id_key[whichlocker])) {
            currentpolicy = thisob;
         }
         else {
            policy = thisob;
         }
      }
      thisob = next_inventory(thisob);
   }
   if ((!policy) && (!currentpolicy)) {
      tell_room(environment(this_object()),
          "Locker attendant says: fukkin liar, man. You ain't gorra " +
          "policy!\n");
      say("Locker attendant fines " + this_player()->query_name() +
          " for attempted fraud.\n",
          this_player());
      write("Locker attendant fines you " + LOCKERCHARGE + " gp " +
            "for attempted fraud.\n");
      log_file(LOGFILE,"fined " + LOCKERCHARGE + " gp for fraud.\n");
      this_player()->add_money(-LOCKERCHARGE);
      revenue_so_far += LOCKERCHARGE;
      return 1;
   }
   if ((!policy) && (currentpolicy)) {
      tell_room(environment(this_object()),
          "Locker attendant shouts: Look you fukkin jerk, yer stuffs here!\n");
      tell_room(environment(this_object()),
          "Locker attendant points to locker " + whichlocker + ".\n");
      log_file(LOGFILE,"pointed out locker.\n");
      return 1;
   }
   tell_room(environment(this_object()),
       "Locker attendant says: Hmmf, bet its fake. Gissa look!\n" +
       "Locker attendant takes policy from " + this_player()->query_name() +
       ".\n");
   move_object(policy,this_object());
   log_file(LOGFILE,
            "policy id: " + policy->query_id_key() + ", " +
            "amount: "    + policy->query_amount() + " gp, " +
            "container: " + policy->query_locker() + ",\n" +
            "customer: "  + policy->query_cust_name() + ".\n");
   if (find_object(policy->query_locker())) {
      log_file(LOGFILE,
               "NOTE: container found by find_object() search,\n");
      value = value_contents(find_object(policy->query_locker()));
      if (value == policy->query_amount()) {
         tell_room(environment(this_object()),
             "Policeman enters.\n" +
             "Policeman says: We have recovered that stolen property, sir.\n" +
             "Policeman gives bag to " + this_player()->query_name() + ".\n");
         log_file(LOGFILE,
                  "and container search matched policy details.\n");
         bag = clone_object("/obj/bag");
         move_contents(find_object(policy->query_locker()),bag);
         move_object(bag, this_player());
         destruct(policy);
         return 1;
      }
      log_file(LOGFILE,
               "but contents did not match policy details.\n");
   }
   tell_room(environment(this_object()),
       "Locker attendant says: fukkit, I gotta pay ya..\n");
   tell_room(environment(this_object()),
       "Locker attendant gives " + this_player()->query_name() +
       " " + policy->query_amount() + " gold coins.\n");
   if (policy->query_amount() < 0) {
      tell_room(environment(this_object()),
          "Locker attendant says: Thats odd.. never mind!\n");
   }
   else {
      this_player()->add_money(policy->query_amount());
      revenue_so_far -= policy->query_amount();
   }
   log_file(LOGFILE,
            "Compensation paid, revenue so far is now " + revenue_so_far +
            " gp.\n");
   destruct(policy);
   return 1;
}






put(str) {
   object thisob;
   object nextob;
   string id1;

   if ((!str) ||
       (current_locker == 0) ||
       (customer_name != this_player()->query_real_name())) {
      return 0;
   }

   if (sscanf(str, "all in locke%s", id1) < 1) {
      return 0;
   }

   thisob = first_inventory(this_player());
   while (thisob) {
      nextob = next_inventory(thisob);
      if ((thisob->short()) &&
          (thisob->get())   &&
          (!thisob->drop())) {
         if (!transfer(thisob,this_object())) {
            write("You put " + thisob->short() + " in the locker.\n");
            say(this_player()->query_name() +
                " puts " + thisob->short() + " in the locker.\n",
                this_player());
         }
      }
      thisob = nextob;
   }
   return 1;
}

add_revenue(arg) {
  revenue_so_far += arg;
}


